# audioGuide copyright Benjamin Hackbarth, 2011-2013 #
__author__ = "Benjamin Hackbarth, Norbert Schnell, Philippe Esling, Diemo Schwarz"
__author_email__ = "hackbarth@gmail.com"
__version__ = 1.01

# import standard libraries/functions
import sys, os
try:
	import json as json
except ImportError:
	try:
		import simplejson as json
	except: pass



import bin_config
config = bin_config.get(__version__)
# lib dur for PURE PYTHON MODULES
sys.path.append(os.path.join(os.path.dirname(__file__), 'pylib'))
# lib dur for PRECOMPILED MODULES
sys.path.append(config['py_compiled_lib_dir'])



from time import clock, localtime
import random
from copy import deepcopy
import numpy as np
import userinterface
import parseops
import paths
import metrics
import normalize as norm
import simcalc
import utilities
import sdiflinkage
# include audioguide files
execfile(os.path.join(os.path.dirname(__file__), 'classes.py'))


class main:
	def __init__(self, scriptPath, defaultOptionsPath, loadOptionsFromFile=None, pythonicUserOptions={}, printName=None, optionsPath=None):
		global ag, tgt, outputFiles, ops, settingsFile, src, defaults, p, path
		agPath = os.path.dirname(__file__)
		path = paths.pathRetriever(agPath, scriptPath, loadOptionsFromFile, optionsPath=optionsPath)
		ops = parseops.parseOptions(agPath, scriptPath, loadOptionsFromFile, pythonicUserOptions, defaultOptionsPath)
		if ops.OUTPUT_LOG_FILE == None: log = None
		else: log = path.locate(ops.OUTPUT_LOG_FILE)
		p = userinterface.Printer(ops.VERBOSITY, ops.USE_PROGRESS_BAR, ops.PRINT_LENGTH, log)
		p.middle(printName, 2)
		path.checkDirsExist(ops)
		for module in [simcalc, userinterface, metrics, norm]: 
			setattr(module, 'ops', globals()['ops']) # add ops to module's namespace
			setattr(module, 'p', globals()['p']) # add p.post to module's namespace
			setattr(module, 'path', globals()['path']) # add path to module's namespace






	def init(self, printConfig=True):
		global ag, tgt, outputFiles, ops, settingsFile, src, defaults, path, agData, SdifInterface, descriptorData
		descriptorData = DescriptorManager()

		SdifInterface = sdiflinkage.SdifInterface(ops, path, config['bin_path']['ircamd'], config['bin_path']['pm2'], config['bin_path']['supervp'])
		
		descriptorData.allDescriptorsList(SdifInterface)
		
		metrics.set_all_segmented_methods(descriptorData.allSegmentMethods)

		ag = analData()
		
		for module in [simcalc, userinterface]: setattr(module, 'ag', globals()['ag']) # add ag to each module's namespace
		src = source(ops.CORPUS)
		agData = otherSavedData(path.locate(ops.OUTPUT_DATA_JSON_FILE), "audioguide-data")
		if ops.SAVE_CONFIG_FILE:
			ops.writeConfigurationFile(path.locate(ops.CONFIG_FILE_NAME))
		p.post( "" )
		ops.needToInitalize = False





	def changeOption(self, name, value):
		global ops
		setattr(ops, name, value)
	



	def loadDatabase(self):
		global ag, tgt, outputFiles, ops, settingsFile, src, p, path, descriptorData
		if ops.needToInitalize: self.init()
		startTime = clock()
		if ops.PRINT_CORPUS_INFO: src.printCorpusInfo()
		segmentFileDetect = ''
		ag.timer("corpus", "set")
		p.barOpen("Evaluating Corpus...", len(src.src)+len(src.src))
		#corpusXml = buildCorpusXmlResouces(src.reqXmlFiles)
		prelimHandles = []
		p.post("Parsing Corpus Segments", 3)
		# init each corpus segment's handle
		for segDict in src.src:
			fileHead = os.path.split(segDict['file'])[1]
			if len(fileHead) > 50: # too long to print... truncate it!
				fileHead = fileHead[:15]+'...'+fileHead[-15:]
			p.post(fileHead+"@"+str(utilities.trunc(segDict['startSec'], 2)), inc=True)
			sf = addSfSegment(segDict) # init each segment
			sf.initAnal(descriptorData)
			# write xml data if all descriptors of all segments are analyzed
			#corpusXml.checkAndWriteXmlResouces(segDict['XmlFile'])
			prelimHandles.append(sf)
		for h in prelimHandles: SdifInterface.doneSdifResource(h, h.filename)
		# run limitations if called for..
		utilities.testCorpusLimits(prelimHandles, descriptorData.corpus_limit, p, ag)
		# initialize all descriptors for segments which passed the limit descriptors
		for sf in prelimHandles:
			sf.loadSegmentPostAnal() # if passed limitAnals
		p.barClose("Read "+str(len(ag.sourceHandles))+"/"+str(src.preLimitTotal)+" segments ("+str(int(len(ag.sourceHandles)/float(src.preLimitTotal)*100))+"%, "+utilities.trunc(ag.corpusLengthInSeconds/60., 2)+" min.).")
		# test to make sure some samples made it though... 
		if len(ag.sourceHandles) == 0:
			print "No database segments made it into the selection pool.  Check limitations.."
			utilities.exit(1)
		if len(ops.ORDER_CORPUS_BY_DESCRIPTOR) > 0: self.renderCorpusOrderSfAndLabels()
		ops.needToLoadSrc = False	
		ag.timer("corpus", "get")

		




	def renderCorpusOrderSfAndLabels(self, tempo=144, gapBetweenSounds=0.2):
		for descriptor in ops.ORDER_CORPUS_BY_DESCRIPTOR:
			tmp = []
			for h in ag.sourceHandles: tmp.append((h.data['raw'][descriptor][0], h))
			tmp.sort() # get ordering
			scoTime = 0
			notes = []
			labels = []
			allTimes = []
			for val, h in tmp:
				allTimes.append((h.startSec, h.endSec, h.filename))
				dur = (h.endSec-h.startSec)
				notes.append(['i1', str(scoTime), str(dur), "100", '"'+os.path.abspath(h.filename)+'"', str(h.startSec), "1", str(utilities.ampToDb(h.data['raw']['power-seg'][0])), str(h.data['raw']['peakTime-seg'][0]), str(h.data['raw']['effDur-seg'][0]), '0.01', '0.01', str(int(h.voiceID)), '0'])
				labels.append((str(scoTime), str(scoTime+dur), str(h.data['raw'][descriptor][0]), h.name))
				if tempo == 'cpsDur':
					scoTime += dur+gapBetweenSounds
				elif type(tempo) in [float, int]:
					scoTime += 60./float(tempo)
			
			tmpsco = os.path.join(ops.TMP_PATH, 'tmpOrderingScore.sco')
			tmplabels = path.locate(os.path.join(os.path.split(ops.CSOUND_OUTPUT_FILE)[0], 'corpus-ordered-'+descriptor+'.txt'))
			scoh = open(tmpsco, 'w')
			labelsh = open(tmplabels, 'w')
			for note in notes:
				scoh.write(' '.join(note)+'\n')
			for label in labels:
				labelsh.write('\t'.join(label)+'\n')
			scoh.close()
			labelsh.close()
			outputPath = path.locate(os.path.join(os.path.split(ops.CSOUND_OUTPUT_FILE)[0], 'corpus-ordered-'+descriptor+'.aiff'))
			renderWithCsound([csoundRenderObj(tmpsco, outputPath, len(notes))], "Rendering corpus ordered by "+descriptor)






	def loadTarget(self):
		global ag, tgt, outputFiles, ops, settingsFile, src, descriptorData
		if ops.needToInitalize:
			self.init()
		ag.timer("target", "set")
		# test files and die if missing
		path.testAndDie(ops.TARGET[0], ops.TARGET_PATH)
		shortName = os.path.split(ops.TARGET[0])[1]	
		tgt = targetData(ops.TARGET[0], ops.TARGET[1], ops.TARGET[2]) # load the target soundfile; do metrics
		p.barOpen('Evaluating Target "'+shortName+'" from '+str(ops.TARGET[1])+"->"+str(ops.TARGET[2]), 1+3)
		p.post(shortName, inc=True)
		tgt.initAnal(descriptorData)
		tgt.data['orig'] = deepcopy(tgt.data['raw']) # preserve a copy of original data
		tgt.onsetInit()
		tgt.offsetInit()
		tgt.initMixture()		
		tgt.segs = tgt.getSegs(path) # dont need first times
		tgt.initSegMethod(descriptorData.target_seg)
		p.barClose("Done.")
		#if not ops.SEGMENTATION_RUN: tgt.xmlResource.writeXmlData()
		if ops.MAKE_OPEN_MUSIC_SCORE: tgt.writeOMScore(path.locate(ops.TGT_OM_SCORE_NAME), tgt.segs)
		if ops.SAVE_TARGET_SEGMENT_LABELS: tgt.writeSegLabelsToDisk(path.locate(ops.TARGET_SEGMENT_LABELS), tgt.segs, ops.TEMPO_CHANGE)
		for module in [simcalc, userinterface]: setattr(module, 'tgt', globals()['tgt']) # add tgt to each module's namespace
		ops.needToLoadTgt = False		
		ops.needToRewindTgt = False
		ag.timer("target", "get")
		if ops.TARGET_PLOT_METRICS:
			for d in descriptorData.master_list:
				if d.weight <= 0: continue
				print "\nplotting", d.name, 
				#print descript, list(tgt.data['raw'][descript])
				plotData = utilities.makeDescriptorGraphable(d.name, tgt)
				outputName = "/Users/ben/Desktop/"+d.name+".png"
				plot(outputName, [tgt.data['raw']['power'], plotData], ['power', descript], segs=False, height=5, normalize=True)

#			names = ['power']
#			ploters = [tgt.data['raw']['power']]
#			for descript in  ['mfcc12', 'mfcc12', 'mfcc14', 'mfcc15', 'mfcc16', 'mfcc17', 'mfcc18', 'mfcc19']:
#				names.append(descript)
#				ploters.append( utilities.makeDescriptorGraphable(descript, tgt) )
#			outputName = "/Users/ben/Desktop/centroidsub-segs.png"
#			plot(outputName, ploters, names, segs=False, height=5, normalize=True)





	def normalizeData(self):
		global ag, tgt, outputFiles, ops, settingsFile, src, agData, SdifInterface
		p.post("Normalizing Data By", ops.NORMALIZE_METHOD,"..", 4)
		ag.timer("norm", "set")
		# normalize raw data..
		#SdifInterface.makeClusterAnalysis(descriptorData, tgt, ag.sourceHandles, 30)
		
		
		
		
		if ops.NORMALIZE_METHOD in ['minmax', 'stddev']:
			ag.targetNormCoeffs, ag.sourceNormCoeffs = norm.normalizeCoefs(descriptorData.normalize_compute, tgt, ag, p, ops.NORMALIZE_METHOD)
			norm.normalizeDescriptors([tgt], descriptorData.normalize_compute, 'raw', 'norm', ops.NORMALIZE_METHOD, ag.targetNormCoeffs, None, None) 
			norm.normalizeDescriptors(ag.sourceHandles, descriptorData.normalize_compute, 'raw', 'norm', ops.NORMALIZE_METHOD, ag.sourceNormCoeffs, None, None) 
			if ops.PRINT_NORM_INFO: ag.postNormalizePrint()
		# normalize with clustering algorithm
		elif ops.NORMALIZE_METHOD == 'timbreRemap':		
			# now reevaluate the target's normalized space according to the corpus' distribution
			evalTgtSegments, notRemapDescripts = norm.targetTimbreReassignment(descriptorData, tgt, ag.sourceHandles, 'raw', agData)
			tgt.initSegMethod(evalTgtSegments)
			# any feature modification happens BEFORE the remapping!
			#tgt.featureModification(None, None, 'raw') # processes whatever is in ops.TARGET_DESCRIPTORS_MOD
			# normalize all metrics with the same coeffs (now ignore norm #2)
			ag.targetNormCoeffs, ag.sourceNormCoeffs = norm.normalizeCoefs(descriptorData.normalize_compute, tgt, ag, p, 'stddev')
			allNormCoeffs = {}
			for d in descriptorData.normalize_compute:
				allNormCoeffs[d.name] = norm.getNormCoefs(ag.allHandles, d.name, None, None, 'stddev')

			norm.normalizeDescriptors([tgt], descriptorData.normalize_compute, 'raw', 'norm', 'stddev', allNormCoeffs, None, None) 
			norm.normalizeDescriptors(ag.sourceHandles, descriptorData.normalize_compute, 'raw', 'norm', 'stddev', allNormCoeffs, None, None) 





#
#		# normalize with clustering algorithm
#		elif ops.NORMALIZE_METHOD == 'cluster':			
#			norm.normalizeByCluster(analMetrics, tgt, ag, agData)
#		tgt.featureModification(None, None, 'norm') # processes whatever is in ops.TARGET_DESCRIPTORS_MOD
#		elapsedTime = ag.timer("norm", "get")
#		agData.close()
#		ops.needToNormalize = False		








	def concatenate(self):
		global ag, tgt, outputFiles, ops, settingsFile, src, descriptorData
		# each of these checks insure that the needed analysis routines have been run before concatenation
		if ops.needToInitalize:
			self.init()
		if ops.needToRemakeAnalLists: descriptorData = DescriptorManager()			
		if ops.needToLoadTgt: # need to initalize the target soundfile?
			self.loadTarget()
		if ops.needToLoadSrc: # need to init and load the corpus?
			self.loadDatabase()
		if ops.needToNormalize:
			self.normalizeData()
		if ops.needToRewindTgt: # need to rewind target -- i.e. restore the data for an additional concate call?
			tgt.rewind()
		random.seed(ops.RANDOM_SEED)
		outputFiles = outputData(ops.CSOUND_SCORE_PATH, ops.CSOUND_MAKE_SCORE, path.locate(ops.OUTPUT_QLIST_NAME), ops.MAKE_QLIST, tgt, path.locate(ops.CSOUND_OUTPUT_FILE), path.locate(ops.OM_SCORE_NAME), ops.MAKE_OPEN_MUSIC_SCORE, path.locate(ops.MIDI_FILE_NAME), ops.MAKE_MIDIFILE)
		src.initConcatenate(tgt.lengthInFrames, ag.sourceHandles)
		src.initTimeVaryingRestrict()
		ag.timer("concat", "set")
		tgt.nextIncrementLength = 1
		######################
		searchObj = simcalc.distanceCalculations(descriptorData)
		#searchObj.makeCorpusCombinations(3, ag.sourceHandles)
		if ops.MATCH_SEARCH_METHOD == "time": self.NEWconcatenateTime(searchObj)
		elif ops.MATCH_SEARCH_METHOD == "power": errors = self.concatenatePower(searchObj)
		if ops.SEARCH_CORPUS_PARAM_INTERPOLATE:
			searchObj.paramInterpolateScore(os.path.split(path.locate(ops.CSOUND_SCORE_PATH))[0])
		searchObj.close()
		#######################
#		if True:
#			searchXml = otherXMLData(path.locate(ops.OUTPUT_DATA_XML_FILE), "searchData")
#			searchObj.writeXml(searchXml)
		
		elapsedTime = np.clip(ag.timer("concat", "get"), 0.0001, sys.maxint)
		p.barClose("Picked "+utilities.bold(str(outputFiles.totalOutputNotes))+" notes in "+str(utilities.trunc(elapsedTime, 2))+" sec. ("+str(utilities.trunc((tgt.lengthInFrames*ag.frameSec)/elapsedTime, 2))+"x realtime)")
		outputFiles.done(ops.CSOUND_MAKE_SCORE, ops.MAKE_QLIST, ops.MAKE_OPEN_MUSIC_SCORE, ops.MAKE_MIDIFILE)
		if True:
			sumTerminations = 0.0
			for key in ag.logicForSegmentTermination: sumTerminations += ag.logicForSegmentTermination[key]
			if sumTerminations != 0:
				for key in ag.logicForSegmentTermination: ag.logicForSegmentTermination[key] = (ag.logicForSegmentTermination[key]/sumTerminations)*100
				utilities.printDict("segment termination logic", ag.logicForSegmentTermination, 2, p)


		if ops.PRINT_SELECTION_HISTO: src.printSampleHistogram()
		if ops.PRINT_SIM_SELECTION_HISTO: src.printSimultanousHistogram()
		# other output files
		if ops.SAVE_TARGET_SEGMENT_LABELS:
			tgt.writeSegLabelsToDisk(path.locate(ops.TARGET_SEGMENT_MULTI_LABELS), tgt.selectedSegs, ops.TEMPO_CHANGE)
		if ops.MAKE_OPEN_MUSIC_SCORE:
			outputFiles.writeOMScoreNote()
		if ops.MAKE_ENP_SCORE:
			outputFiles.writePWGLScoreNote(path.locate(ops.ENP_SCORE_NAME))
		if ops.MAKE_BACH_SCORE:
			outputFiles.writeBachScoreNotes(path.locate(ops.BACH_SCORE_NAME), ag.sourceHandles, path.locate(ops.CSOUND_OUTPUT_FILE))
		if ops.MAKE_LISP_EVENT_LIST:
			outputFiles.writeLispEvents(path.locate(ops.LISP_EVENT_LIST_NAME))
		ops.needToRewindTgt = True

		#for data in searchObj.searchResults:
		#	print data


	def concatenatePower(self, searchObj):
		global ag, tgt, outputFiles, ops, settingsFile, src
		printSelect = False 
		currentSeg = -1
		numbSelectedFrames = 0
		errors = 0.
		selectionsThisSegment = []
		p.barOpen('Concatenating Based on '+ops.MATCH_SEARCH_METHOD, len(tgt.segFrames)+1)
		powerSortedSegList = tgt.initSegmentList(tgt.segFrames)
		
		while len(powerSortedSegList) > 0:
			power, start, end, thisPeakOffset = powerSortedSegList[0]
			tgt.timeInFrames = start
			thisSegmentLength = end-start
#			matchThisFrame,reason = tgt.segmentSuperimpositionTest(tgt.timeInFrames, tgt.segmentsPerFrame, ops.MIN_SIMULTANEOUS_SEGMENTS, ops.MAX_SIMULTANEOUS_SEGMENTS, tgt.onsetsPerFrame, ops.MIN_SEGMENTS_PER_ONSET, ops.MAX_SEGMENTS_PER_ONSET) 
#			if not matchThisFrame: # if its not time to pick a new sample...
#				tgt.increment(ops.TARGET_INCREMENT) # increment
#				ag.logicForSegmentTermination[reason] += 1
#				continue
			ag.activeSegments = ag.updateActiveSegments(tgt.timeInFrames, thisSegmentLength)
			if ops.MAX_SIMULTANEOUS_SEGMENTS not in [None, "None"]:
 				if len(ag.activeSegments) > ops.MAX_SIMULTANEOUS_SEGMENTS:
					del powerSortedSegList[0]
					errors += ag.calculatePostSelectionError(start, end, selectionsThisSegment)
					selectionsThisSegment = []
					p.post('', inc=True, forceInc=len(tgt.segFrames)-tgt.lengthOriginalSegs)
 					ag.logicForSegmentTermination['max overlapping segments'] += 1
 					continue
			###################################
			## test for max notes this frame ##
			###################################
			if tgt.onsetsPerFrame[tgt.timeInFrames] >= ops.MAX_SEGMENTS_PER_ONSET: # exit if max notes has been reached
				del powerSortedSegList[0]
				errors += ag.calculatePostSelectionError(start, end, selectionsThisSegment)
				selectionsThisSegment = []
				p.post('', inc=True, forceInc=len(tgt.segFrames)-tgt.lengthOriginalSegs)
				tgt.lengthOriginalSegs -= 1
 				ag.logicForSegmentTermination['max segments per onset'] += 1
				continue
			##### FIND VALID CORPUS HANDLES #####
			validHandles, validSteams = src.evaluateValidSamples(ops.MAX_ACTIVE_VOICES) # build valid handles based on time-varying constraints...
			if len(validHandles) == 0: # nothing valid, so skip to new frame...
				p.post("Out of Samples", 3)
				ag.countRanOutOfSamples += 1
				del powerSortedSegList[0]
				errors += ag.calculatePostSelectionError(start, end, selectionsThisSegment)
				selectionsThisSegment = []
				tgt.lengthOriginalSegs -= 1
 				ag.logicForSegmentTermination['out of valid samples'] += 1
				continue # move on, nothing available here!
			##### SEARCH AVAILBLE CORPUS SAMPLES #####
			searchObj.setCorpus(validHandles, validSteams)
			searchObj.filterCorpus(tgt, tgt.timeInFrames, thisSegmentLength, thisPeakOffset)

			searchObj.executeSearch(descriptorData.simliarity_search_steps, tgt.timeInFrames, thisSegmentLength, thisPeakOffset, tgt.segmentsPerFrame[tgt.timeInFrames], len(tgt.segFrames)-tgt.lengthOriginalSegs) # adds result to searchObj.searchResults

			chosenSample, chosenData, searchStats = searchObj.searchResults[-1]
			##### MODIFY CHOSEN SAMPLES AMPLITUDE? #####
			if ops.POST_SELECTION_AMP_SCALE in ["lstSqr", "lstsqr"]:
				sourceAmpScale, outputDb = tgt.leastSqrsAmp(chosenSample, ops.POST_SELECTION_AMP_MIN, ops.POST_SELECTION_AMP_MAX, chosenData['segmentSt'], chosenData[ops.CORPUS_AMPLITUDE_DURATION])
			elif ops.POST_SELECTION_AMP_SCALE in ["peakAmp"]:
				sourceAmpScale, outputDb = tgt.peakAmpCorrect(chosenSample, ops.POST_SELECTION_AMP_MIN, ops.POST_SELECTION_AMP_MAX, chosenData['segmentSt'], chosenData[ops.CORPUS_AMPLITUDE_DURATION])
			elif ops.POST_SELECTION_AMP_SCALE in ["averageAmp"]:
				sourceAmpScale, outputDb = tgt.averagedAmpCorrect(chosenSample, ops.POST_SELECTION_AMP_MIN, ops.POST_SELECTION_AMP_MAX, chosenData['segmentSt'], chosenData[ops.CORPUS_AMPLITUDE_DURATION])
			else:
				sourceAmpScale, outputDb = (1, 100)
 			##### append the one that just got chosen..
			ag.selectedSegments.append((chosenData['simCalcStEnd'][0], chosenData['simCalcStEnd'][0]+chosenSample.lengthInFrames, chosenSample, sourceAmpScale))
			##### subtract power to update onset detection
			tgt.subtractPowerForOnsetEnvelope(chosenSample, sourceAmpScale, chosenData['segmentSt'], int(chosenData[ops.CORPUS_SUBTRACT_DURATION]))
			##### mix chosen sample's descriptors into a tgt dict that holds cumulative mixtures..
			if ops.SIMULTANEOUS_SELECTION_METHOD == "mixture":
				tgt.mixSelectedSamplesDescriptors(chosenSample, sourceAmpScale, chosenData['segmentSt'], chosenData[ops.CORPUS_SUBTRACT_DURATION])
			#####
			powerSortedSegList = tgt.updateSegmentList(powerSortedSegList, start, end)
			selectionsThisSegment.append(chosenSample)
			#####
			tgt.selectedSegs.append((tgt.timeInFrames*ag.frameSec, thisSegmentLength*ag.frameSec, tgt.lastSegmentInitiated, tgt.lastSegmentTerminated))
			trans = utilities.getTransposition(tgt, chosenSample, thisSegmentLength, ops.TRANSPOSE_QUANTIZE)
			if chosenData['trans'] != 0.: trans = chosenData['trans']
			thisSeg = tgt.getCurrentSegNumber(tgt.timeInFrames, tgt.segFrames)
			##### manage loudestPeak descriptor
			if tgt.data['raw'].has_key('loudestPeak'):
				currentPeak = int(np.median(tgt.segmentsPerFrame[tgt.timeInFrames:tgt.timeInFrames+thisSegmentLength]))+1
				tgt.data['raw']['loudestPeak'][tgt.timeInFrames:tgt.timeInFrames+thisSegmentLength] = tgt.data['raw']['peakFrq-'+str(currentPeak%ops.NUMBER_OF_SPECTRAL_PEAKS)][tgt.timeInFrames:tgt.timeInFrames+thisSegmentLength]



			if thisSeg != currentSeg: # for writing score comments!
				outputFiles.addComment([';tgt-segment', str(tgt.segs[thisSeg][0]), str(tgt.segs[thisSeg][1]), str(thisSeg), '\n'])
				currentSeg = thisSeg
			# get note duration
			if ops.CSOUND_SCORE_DURATION_TYPE == 'corpusDur':
				durSec = chosenSample.endSec-chosenSample.startSec
			else:
				durSec = chosenData[ops.CSOUND_SCORE_DURATION_TYPE]*ag.frameSec
			outputFiles.addEvent(chosenSample, utilities.limit(chosenData['segmentSt']*ag.frameSec, 0, None), durSec, chosenData['targetDur'], outputDb, trans, thisSeg)


			minSelectedDuration = min(thisSegmentLength, chosenData['corpusEffDur'])
			tgt.segmentsPerFrame[tgt.timeInFrames:tgt.timeInFrames+minSelectedDuration] += 1 ##### JUST CHANGED!
			if ops.CORPUS_POCKETING:
				src.updateTargetPocketCorpusRestriction(tgt.timeInFrames, tgt.segFrames, chosenSample)
				
				
			tgt.onsetsPerFrame[tgt.timeInFrames] += 1
			src.corpusVcOverlapCounter[chosenSample.voiceID][tgt.timeInFrames:tgt.timeInFrames+chosenData[ops.CORPUS_OVERLAP_DURATION]] += 1
			tgt.setNextInc(ops.TARGET_INCREMENT, thisSegmentLength, chosenData['corpusEffDur'])
			src.lastVoice = chosenSample.voiceID
			src.updateSourcePool(chosenSample)
			src.updateRestrict(chosenSample, chosenData)
			#if len(ag.activeSegments) == 0:
			#	plot(path.locate("output", 'mix-'+str(tgt.timeInFrames)+'-'+str(len(ag.activeSegments))+'.png'), [tgt.data['mix-raw']['mfcc1'], tgt.data['orig']['mfcc1']], ["mix", "orig"], segs=False, normalize=False, height=12, width=45)
		return errors








#	def concatenateTime(self, searchObj):
#		global ag, tgt, outputFiles, ops, settingsFile, src
#		currentSeg = -1
#		numbSelectedFrames = 0
#		p.barOpen('Concatenating...', int(tgt.lengthInFrames))
#		while tgt.timeInFrames < tgt.lengthInFrames-1:
#			########################################
#			## test for max simultaneous segments ##
#			########################################
#			if ops.MAX_SIMULTANEOUS_SEGMENTS not in [None, "None"]:
#				ag.updateActiveSegments(tgt.timeInFrames, 1)
#				ag.activeSegments = ag.updateActiveSegments(tgt.timeInFrames, tgt.timeInFrames+1)
# 				if len(ag.activeSegments) > ops.MAX_SIMULTANEOUS_SEGMENTS:
#					tgt.increment(ops.TARGET_INCREMENT) # increment
#					p.post('', inc=True, forceInc=tgt.timeInFrames)
# 					ag.logicForSegmentTermination['max overlapping segments'] += 1
#					continue
#			###################################
#			## test for max notes this frame ##
#			###################################
#			if tgt.onsetsPerFrame[tgt.timeInFrames] >= ops.MAX_SEGMENTS_PER_ONSET: # exit if max notes has been reached
#				tgt.increment(ops.TARGET_INCREMENT) # increment
#				ag.logicForSegmentTermination['max segments per onset'] += 1
#				#p.post('', inc=True, forceInc=tgt.timeInFrames)
#				continue
#			#################################
#			## test tgt for trigger thresh ##
#			#################################
#			matchThisFrame = tgt.thresholdTest(tgt.timeInFrames) 
#			if not matchThisFrame: # if its not time to pick a new sample...
#				tgt.increment(ops.TARGET_INCREMENT) # increment
#				ag.logicForSegmentTermination['below onset threshold'] += 1
#				#p.post('', inc=True, forceInc=tgt.timeInFrames)
#				continue
#			#######################
#			## build sample pool ##
#			#######################
#			validHandles, validSteams = src.evaluateValidSamples(ops.MAX_ACTIVE_VOICES) # build valid handles based on time-varying constraints...
#			if len(validHandles) == 0: # nothing valid, so skip to new frame...
#				p.post("Out of Samples", 3)
#				ag.countRanOutOfSamples += 1
#				tgt.increment(ops.TARGET_INCREMENT) # increment
#				ag.logicForSegmentTermination['out of valid samples'] += 1
#				#p.post('', inc=True, forceInc=tgt.timeInFrames)
#				continue # move on, nothing available here!
#			##################################################
#			# if passing this point, a sample will be picked #
#			##################################################
#			if tgt.segmentsPerFrame[tgt.timeInFrames] == 0 and ops.CSOUND_WRITE_SCORE_COMMENTS:
#				outputFiles.scoreNotes.append([';tgt-frameNumber', str(numbSelectedFrames), '\n'])
#				numbSelectedFrames += 1
#			thisSegmentLength = tgt.findRawSegmentLength(tgt.timeInFrames)
#			thisPeakOffset = metrics.peakTimeSeg(tgt.data[ops.CONCATENATE_PEAK_ALIGN_ENVELOPE]['power'][tgt.timeInFrames:tgt.timeInFrames+thisSegmentLength])[0]
#			ag.activeSegments = ag.updateActiveSegments(tgt.timeInFrames, thisSegmentLength)
#
#			########################
#			## brute force search ##
#			########################
#			searchObj.setCorpus(validHandles, validSteams)
#			searchObj.filterCorpus(tgt, tgt.timeInFrames, thisSegmentLength, thisPeakOffset)
#			if len(ops.FIRST_PASS_DESCRIPTORS) != 0:
#				searchObj.searchCorpus('FIRST_PASS_DESCRIPTORS', tgt.timeInFrames, thisSegmentLength, thisPeakOffset, tgt.segmentsPerFrame[tgt.timeInFrames], completeResults=True, forceInc=tgt.timeInFrames)
#				searchObj.retainPercentage(ops.FIRST_PASS_PERCENT)
#			searchObj.searchCorpus('DESCRIPTORS', tgt.timeInFrames, thisSegmentLength, thisPeakOffset, tgt.segmentsPerFrame[tgt.timeInFrames], completeResults=ops.ALWAYS_MAKE_COMPLETE_MATCHING_RESULTS, forceInc=tgt.timeInFrames)
#			chosenSample, chosenSampleData = searchObj.returnSampleHandle(ops.MATCH, ops.MATCH_PERCENT)			
#			# add the chosen segment to pool.
#			# samples are removed when they expire.
#			# keep in case existing segments are accounted for in matching..
#			if ops.PLOT_MATCHES: 
#				utilities.plotMatches(tgt, chosenSample, chosenSampleData[2]['simCalcStEnd'][0], 0, thisPeakOffset, chosenSample.data['raw']['peakTime-seg'][0], chosenSampleData[2]['simCalcStEnd'][1]-chosenSampleData[2]['simCalcStEnd'][0], str(utilities.trunc(tgt.timeInFrames*ag.frameSec, 2))+'-'+str(tgt.notesThisFrame), 'raw', metrics=descriptorData.master_list, normalize=False)
#			###################################
#			## MANAGE SIMULTANEOUS SELECTION ##
#			###################################
#			if ops.POST_SELECTION_AMP_SCALE in ["lstSqr", "lstsqr"]:
#				sourceAmpScale, outputDb = tgt.leastSqrsAmp(chosenSample, ops.POST_SELECTION_AMP_MIN, ops.POST_SELECTION_AMP_MAX, chosenSampleData[2]['segmentSt'], chosenSampleData[2][ops.CORPUS_AMPLITUDE_DURATION])
#			else:
#				sourceAmpScale, outputDb = (1, 100)
# 			# append the one that just got chosen..
#			ag.selectedSegments.append((chosenSampleData[2]['simCalcStEnd'][0], chosenSampleData[2]['simCalcStEnd'][0]+chosenSample.lengthInFrames, chosenSample, sourceAmpScale))
#
#			# subtract power to update onset detection
#			tgt.subtractPowerForOnsetEnvelope(chosenSample, sourceAmpScale, chosenSampleData[2]['segmentSt'], int(chosenSampleData[2][ops.CORPUS_SUBTRACT_DURATION]))
#			# mixture
#			if ops.SIMULTANEOUS_SELECTION_METHOD == "mixture":
#				tgt.mixSelectedSamplesDescriptors(chosenSample, sourceAmpScale, chosenSampleData[2]['segmentSt'], chosenSampleData[2][ops.CORPUS_SUBTRACT_DURATION])
#			#####################################
#			## WRITE NOTES AND UPDATE COUNTERS ##
#			#####################################
#			tgt.selectedSegs.append((tgt.timeInFrames*ag.frameSec, thisSegmentLength*ag.frameSec, tgt.lastSegmentInitiated, tgt.lastSegmentTerminated))
#			trans = utilities.getTransposition(tgt, chosenSample, thisSegmentLength, ops.TRANSPOSE_QUANTIZE)
#			note = [utilities.limit(chosenSampleData[2]['segmentSt']*ag.frameSec, 0, None), outputDb, chosenSample, None, trans, chosenSample.data['raw']['peakTime-seg'][0]*ag.frameSec, chosenSample.data['raw']['effDur-seg'][0]*ag.frameSec] # note entry
#			thisSeg = tgt.getCurrentSegNumber(tgt.timeInFrames, tgt.segFrames)
#
#			# manage loudestPeak descriptor
#			if tgt.data['raw'].has_key('loudestPeak'):
#				currentPeak = int(np.median(tgt.segmentsPerFrame[tgt.timeInFrames:tgt.timeInFrames+thisSegmentLength]))+1
#				tgt.data['raw']['loudestPeak'][tgt.timeInFrames:tgt.timeInFrames+thisSegmentLength] = tgt.data['raw']['peakFrq-'+str(currentPeak%ops.NUMBER_OF_SPECTRAL_PEAKS)][tgt.timeInFrames:tgt.timeInFrames+thisSegmentLength]
#
#
#			if thisSeg != currentSeg: # for writing score comments!
#				outputFiles.addComment([';tgt-segment', str(tgt.segs[thisSeg][0]), str(tgt.segs[thisSeg][1]), str(thisSeg), '\n'])
#				currentSeg = thisSeg
#
#			outputFiles.addEvent(chosenSample, utilities.limit(chosenSampleData[2]['segmentSt']*ag.frameSec, 0, None), chosenSampleData[2][ops.CSOUND_SCORE_DURATION_TYPE], chosenSampleData[2]['targetDur'], outputDb, trans, thisSeg)
#
#			tgt.segmentsPerFrame[tgt.timeInFrames:tgt.timeInFrames+thisSegmentLength] += 1
#			tgt.onsetsPerFrame[tgt.timeInFrames] += 1
#			tgt.setNextInc(ops.TARGET_INCREMENT, thisSegmentLength, chosenSampleData[2]['corpusEffDur'])
#			src.lastVoice = chosenSample.voiceID
#			src.corpusVcOverlapCounter[chosenSample.voiceID][tgt.timeInFrames:tgt.timeInFrames+chosenSampleData[2][ops.CORPUS_OVERLAP_DURATION]] += 1
#			src.updateSourcePool(chosenSample)
#			src.updateRestrict(chosenSample, chosenSampleData[2])
#			#plot(path.locate("output", 'mix-'+str(tgt.timeInFrames)+'-'+str(len(ag.activeSegments))+'.png'), [tgt.data['mix-raw']['mfcc1'], tgt.data['orig']['mfcc1']], ["mix", "orig"], segs=False, normalize=True, height=12, width=45)
#
#










	def NEWconcatenateTime(self, searchObj):
		global ag, tgt, outputFiles, ops, settingsFile, src
		currentSeg = -1
		numbSelectedFrames = 0
		p.barOpen('Concatenating...', int(tgt.lengthInFrames))
		while tgt.timeInFrames < tgt.lengthInFrames-1:
			#################################
			## test tgt for trigger thresh ##
			#################################
			matchThisFrame,reason = tgt.segmentSuperimpositionTest(tgt.timeInFrames, tgt.segmentsPerFrame, ops.MIN_SIMULTANEOUS_SEGMENTS, ops.MAX_SIMULTANEOUS_SEGMENTS, tgt.onsetsPerFrame, ops.MIN_SEGMENTS_PER_ONSET, ops.MAX_SEGMENTS_PER_ONSET) 
			if not matchThisFrame: # if its not time to pick a new sample...
				tgt.increment(ops.TARGET_INCREMENT) # increment
				ag.logicForSegmentTermination[reason] += 1
				continue
			#######################
			## build sample pool ##
			#######################
			validHandles, validSteams = src.evaluateValidSamples(ops.MAX_ACTIVE_VOICES) # build valid handles based on time-varying constraints...
			if len(validHandles) == 0: # nothing valid, so skip to new frame...
				p.post("Out of Samples", 3)
				ag.countRanOutOfSamples += 1
				tgt.increment(ops.TARGET_INCREMENT) # increment
				ag.logicForSegmentTermination['out of valid samples'] += 1
				#p.post('', inc=True, forceInc=tgt.timeInFrames)
				continue # move on, nothing available here!
			##################################################
			# if passing this point, a sample will be picked #
			##################################################
			if tgt.segmentsPerFrame[tgt.timeInFrames] == 0 and ops.CSOUND_WRITE_SCORE_COMMENTS:
				outputFiles.scoreNotes.append([';tgt-frameNumber', str(numbSelectedFrames), '\n'])
				numbSelectedFrames += 1
			thisSegmentLength = tgt.findRawSegmentLength(tgt.timeInFrames)
			thisPeakOffset = metrics.peakTimeSeg(tgt.data[ops.CONCATENATE_PEAK_ALIGN_ENVELOPE]['power'][tgt.timeInFrames:tgt.timeInFrames+thisSegmentLength])[0]
			ag.activeSegments = ag.updateActiveSegments(tgt.timeInFrames, thisSegmentLength)
			########################
			## brute force search ##
			########################
			searchObj.setCorpus(validHandles, validSteams)
			searchObj.filterCorpus(tgt, tgt.timeInFrames, thisSegmentLength, thisPeakOffset)

			searchObj.executeSearch(descriptorData.simliarity_search_steps, tgt.timeInFrames, thisSegmentLength, thisPeakOffset, tgt.segmentsPerFrame[tgt.timeInFrames], tgt.timeInFrames) # adds result to searchObj.searchResults

			chosenSample, chosenData, searchStats = searchObj.searchResults[-1]
			# add the chosen segment to pool.
			# samples are removed when they expire.
			# keep in case existing segments are accounted for in matching..
			if ops.PLOT_MATCHES: 
				utilities.plotMatches(tgt, chosenSample, chosenData['simCalcStEnd'][0], 0, thisPeakOffset, chosenSample.data['raw']['peakTime-seg'][0], chosenData['simCalcStEnd'][1]-chosenData['simCalcStEnd'][0], str(utilities.trunc(tgt.timeInFrames*ag.frameSec, 2))+'-'+str(tgt.notesThisFrame), 'raw', metrics=descriptorData.master_list, normalize=False)
			###################################
			## MANAGE SIMULTANEOUS SELECTION ##
			###################################
			if ops.POST_SELECTION_AMP_SCALE in ["lstSqr", "lstsqr"]:
				sourceAmpScale, outputDb = tgt.leastSqrsAmp(chosenSample, ops.POST_SELECTION_AMP_MIN, ops.POST_SELECTION_AMP_MAX, chosenData['segmentSt'], chosenData[ops.CORPUS_AMPLITUDE_DURATION])
			elif ops.POST_SELECTION_AMP_SCALE in ["peakAmp"]:
				sourceAmpScale, outputDb = tgt.peakAmpCorrect(chosenSample, ops.POST_SELECTION_AMP_MIN, ops.POST_SELECTION_AMP_MAX, chosenData['segmentSt'], chosenData[ops.CORPUS_AMPLITUDE_DURATION])
			elif ops.POST_SELECTION_AMP_SCALE in ["averageAmp"]:
				sourceAmpScale, outputDb = tgt.averagedAmpCorrect(chosenSample, ops.POST_SELECTION_AMP_MIN, ops.POST_SELECTION_AMP_MAX, chosenData['segmentSt'], chosenData[ops.CORPUS_AMPLITUDE_DURATION])
			else:
				sourceAmpScale, outputDb = (1, 100)
 			# append the one that just got chosen..
			ag.selectedSegments.append((chosenData['simCalcStEnd'][0], chosenData['simCalcStEnd'][0]+chosenSample.lengthInFrames, chosenSample, sourceAmpScale))

			# subtract power to update onset detection
			tgt.subtractPowerForOnsetEnvelope(chosenSample, sourceAmpScale, chosenData['segmentSt'], int(chosenData[ops.CORPUS_SUBTRACT_DURATION]))
			# mixture
			if ops.SIMULTANEOUS_SELECTION_METHOD == "mixture":
				tgt.mixSelectedSamplesDescriptors(chosenSample, sourceAmpScale, chosenData['segmentSt'], chosenData[ops.CORPUS_SUBTRACT_DURATION])
			#####################################
			## WRITE NOTES AND UPDATE COUNTERS ##
			#####################################
			tgt.selectedSegs.append((tgt.timeInFrames*ag.frameSec, thisSegmentLength*ag.frameSec, tgt.lastSegmentInitiated, tgt.lastSegmentTerminated))
			trans = utilities.getTransposition(tgt, chosenSample, thisSegmentLength, ops.TRANSPOSE_QUANTIZE)
			if chosenData['trans'] != 0.: trans = chosenData['trans']
			note = [utilities.limit(chosenData['segmentSt']*ag.frameSec, 0, None), outputDb, chosenSample, None, trans, chosenSample.data['raw']['peakTime-seg'][0]*ag.frameSec, chosenSample.data['raw']['effDur-seg'][0]*ag.frameSec] # note entry
			thisSeg = tgt.getCurrentSegNumber(tgt.timeInFrames, tgt.segFrames)

			# manage loudestPeak descriptor
			if tgt.data['raw'].has_key('loudestPeak'):
				currentPeak = int(np.median(tgt.segmentsPerFrame[tgt.timeInFrames:tgt.timeInFrames+thisSegmentLength]))+1
				tgt.data['raw']['loudestPeak'][tgt.timeInFrames:tgt.timeInFrames+thisSegmentLength] = tgt.data['raw']['peakFrq-'+str(currentPeak%ops.NUMBER_OF_SPECTRAL_PEAKS)][tgt.timeInFrames:tgt.timeInFrames+thisSegmentLength]


			if thisSeg != currentSeg: # for writing score comments!
				outputFiles.addComment([';tgt-segment', str(tgt.segs[thisSeg][0]), str(tgt.segs[thisSeg][1]), str(thisSeg), '\n'])
				currentSeg = thisSeg

			# get note duration
			if ops.CSOUND_SCORE_DURATION_TYPE == 'corpusDur':
				durSec = chosenSample.endSec-chosenSample.startSec
			else:
				durSec = chosenData[ops.CSOUND_SCORE_DURATION_TYPE]*ag.frameSec
			outputFiles.addEvent(chosenSample, utilities.limit(chosenData['segmentSt']*ag.frameSec, 0, None), durSec, chosenData['targetDur'], outputDb, trans, thisSeg)

			minSelectedDuration = min(thisSegmentLength, chosenData['corpusEffDur'])
			tgt.segmentsPerFrame[tgt.timeInFrames:tgt.timeInFrames+minSelectedDuration] += 1 ##### JUST CHANGED!
			if ops.CORPUS_POCKETING:
				src.updateTargetPocketCorpusRestriction(tgt.timeInFrames, tgt.segFrames, chosenSample)
				
			tgt.onsetsPerFrame[tgt.timeInFrames] += 1
			tgt.setNextInc(ops.TARGET_INCREMENT, thisSegmentLength, chosenData['corpusEffDur'])
			src.lastVoice = chosenSample.voiceID
			src.corpusVcOverlapCounter[chosenSample.voiceID][tgt.timeInFrames:tgt.timeInFrames+chosenData[ops.CORPUS_OVERLAP_DURATION]] += 1
			src.updateSourcePool(chosenSample)
			src.updateRestrict(chosenSample, chosenData)
		#print tgt.onsetsPerFrame
		#print tgt.segmentsPerFrame

		

	def render(self, outputFileName=None):
		#print ag.selectedSegments
		global ag, tgt, outputFiles, ops, settingsFile, src, p
		if not ops.CSOUND_RENDER_SCORE: return False
		ag.timer("render", "set")
		if utilities.which('csound') == None:
			print "\n\tCannot render file because cannot find csound.\n"
			return False

		if outputFileName != None: ag.scoresToRender[0].outputName = outputFileName # user force
		renderWithCsound(ag.scoresToRender, "Rendering "+str(len(ag.scoresToRender))+" file(s) with Csound")
		# other stuff
		if ops.MAKE_OUTPUT_MIX:
			p.post("Making Csound Mix...", 1)
			utilities.executeCommand(['csound --output="'+path.locate(ops.CSOUND_OUTPUT_MIX_FILE)+'"  "'+os.path.join(ops.AG_PATH,'scripts','csoundMix.orc')+'" "'+os.path.join(ops.TMP_PATH, 'agMix.sco')+'" > /tmp/out  2>&1'])
		dateString=str(localtime()[1])+"."+str(localtime()[2])+"-"+str(localtime()[3])+":"+str(localtime()[4])+":"+str(localtime()[5])
		
		if ops.MAKE_SEGMENTED_TARGET_SF:
			p.post("Mixing Target Segments into a Single Soundfile...", 1)
			utilities.executeCommand(['csound --output="'+path.locate(ops.CSOUND_SEGMENTED_TARGET_SF)+' "'+os.path.join(ops.AG_PATH,'scripts','csoundMix.orc')+'" "'+os.path.join(ops.TMP_PATH, 'tgtSegs.sco')+'" > /tmp/out  2>&1'])
		if ops.SAVE_TARGET_SEGMENT_OR_LINK and utilities.which('sox') != None: # must have sox!
			linkName = path.locate(ops.TARGET_SEGMENT_OR_LINK)
			if os.path.exists(linkName): os.remove(os.path.abspath(linkName))
			if ops.TARGET[1] in [0, None, "start", "Start"] and ops.TARGET[2] in [None, "end", "End"]:
				os.symlink(os.path.abspath(tgt.filename), linkName) # just link it
			else:
#				if ops.TARGET[1] in [0, None, "start", "Start"]: start = 0
#				else: start = ops.TARGET[1]
#				if ops.TARGET[2] in [None, "end", "End"]: end = tgt.endSec
#				else: end = ops.TARGET[2]
				utilities.executeCommand(['sox "'+tgt.filename+'" "'+linkName+'" trim '+str(tgt.startSec)+' '+str(tgt.endSec-tgt.startSec)])# clip into a new file
		utilities.executeCommand(['echo "'+ag.scoresToRender[0].outputName+'" > /tmp/last_paplay']) # for my play script...
		ag.timer("render", "get")
		if ops.PRINT_ELAPSED_TIMES: ag.printTimeData()
		ag.scoresToRender = []
		return ops.CSOUND_PLAY_RENDERED_FILE

