import sys, os 
import utilities

class parseOptions:
	def __init__(self, AudioGuidePath, ScriptPath, loadOptionsFromFile, pythonicUserOptions, defaultOptionsPath):
		# audioGuide user options are taken from 2 sources:
		#	from loadOptionsFromFile is it is not None
		#	from pythonicUserOptions
		self.loadOptionsFromFile = loadOptionsFromFile
		self.pythonicUserOptions = pythonicUserOptions
		self.userOps = {}
		if loadOptionsFromFile != None:
			self.userOps.update(file2optionsDict(loadOptionsFromFile))
		if type(pythonicUserOptions) == str:
			# load a string into the options
			self.userOps.update(string2optionsDict(pythonicUserOptions))
		elif type(pythonicUserOptions) == dict:
			# load a dict into the options
			self.userOps.update(pythonicUserOptions)
		# load defaults from disk
		versionDefPath = os.path.join(AudioGuidePath,defaultOptionsPath)
		if not os.path.isfile(versionDefPath):
			utilities.error("ops","The default options file "+versionDefPath+" does not exist!")
		self.defaultOps = file2optionsDict(versionDefPath)
		
		#print self.defaultOps.keys()
		# manage packages
		if self.userOps.has_key('PACKAGES_PATH'): packagesPath = os.path.join(ScriptPath, self.userOps['PACKAGES_PATH'])
		else: packagesPath = os.path.join(ScriptPath, self.defaultOps['PACKAGES_PATH'])
		if self.userOps.has_key('VERBOSITY'): self.v = self.userOps['VERBOSITY']
		else: self.v = self.defaultOps['VERBOSITY']

		self.userOps = self.checkForParameterPackages(self.userOps, packagesPath)
		# get options for correct formatting and type
		checkOptionCasting(self.userOps)
		self.checkAlternativeNames()
		# check options to make sure everything exists
		self.checkOptions()
		self.userOps['AG_PATH'] = AudioGuidePath
		self.userOps['SCRIPT_PATH'] = ScriptPath
		self.userOps['OPS_PATH'] = loadOptionsFromFile
		self.userOps['DEFAULT_PATH'] = versionDefPath
		# these three are kept to that if options are changed we
		# can keep track of what needs to be recomputed.
		self.needToInitalize = True
		self.needToRemakeAnalLists = False
		self.needToLoadTgt = True
		self.needToLoadSrc = True
		self.needToNormalize = True
		self.needToRewindTgt = False
		orderedOps = readOptionOrder(self.userOps['DEFAULT_PATH'])
#		for section in orderedOps:
#			print section[0]
#			for var in section[1:]:
#				print var, str(type(self.defaultOps[var])).split("'")[1], str(self.defaultOps[var])
	############################################################################
	def checkAlternativeNames(self):
		# for compatibility with old versions/names
		renamed = {'SOURCE_NAMES': 'CORPUS', 'SEED': 'RANDOM_SEED', 'SUBTRACTION_SCALE': 'SCALE_POWER_SUBTRACTION', 'DISTANCE_METRICS': 'DESCRIPTORS', 'TARGET_ONSET_METRICS': 'TARGET_ONSET_DESCRIPTORS', 'TARGET_METRIC_MOD': 'TARGET_DESCRIPTORS_MOD', 'TARGET_METRIC_BASH': 'TARGET_DESCRIPTORS_BASH', 'FIRST_PASS_METRICS': 'FIRST_PASS_DESCRIPTORS', 'SOURCE_RESTRICT_FEATURE_IN_TIME': 'CORPUS_RESTRICT_DISCRIPTOR_IN_TIME', 'SOURCE_RESTRICT_NUMBER_STREAMS': 'CORPUS_RESTRICT_NUMBER_STREAMS', 'SCALE_STD_DEV': 'NORMALIZE_SCALE_STD_DEV', 'LIMIT_OUTPUT_DURATIONS': 'LIMIT_SEGMENT_DURATIONS', 'ANAL_LIMITS': 'LIMIT_CORPUS_BY_DESCRIPTOR', 'CAN_USE_SAME_SAMPLE_TWICE': 'CAN_USE_SAME_SEGMENT_TWICE', 'CAN_REUSE_SAMPLE_AFTER': 'CAN_REUSE_SEGMENT_AFTER', 'SOURCE_SUBTRACT_DURATION': 'CORPUS_SUBTRACT_DURATION', 'SOURCE_AMPLITUDE_DURATION': 'CORPUS_AMPLITUDE_DURATION', 'SCALE_TARGET_AMP': 'TARGET_SCALE_AMP'} 
		remapped = {'SUBTRACT': {True: ('SIMULTANEOUS_SELECTION_METHOD', 'subtract'), False: ('SIMULTANEOUS_SELECTION_METHOD', 'None')}}

		for nameDict in [self.userOps, self.defaultOps]:
			dictAdd = {}
			dictModify = {}
			for item in nameDict:
				# do renamed
				if renamed.has_key(item): # renamed
					dictAdd[renamed[item]] = item
				# do remapped
				if remapped.has_key(item): # renamed
					dictModify[remapped[item][nameDict[item]][0]] = remapped[item][nameDict[item]][1], item
			# now change names for each dict
			for newName, (value, oldName) in dictModify.iteritems():
				nameDict[newName] = value
				del nameDict[oldName] # remove the old one
			# now change names for each dict
			for key, val in dictAdd.iteritems():
				nameDict[key] = nameDict[val]
				del nameDict[val] # remove the old one
				
	############################################################################
	def checkOptions(self):
		exceptions = ['TARGET', 'CORPUS', 'DESCRIPTORS']
		for item in self.userOps:
			if not self.defaultOps.has_key(item) and item not in exceptions:
				utilities.error('ops', 'AudioGuide does not have an option called '+item+'!')
	############################################################################
	def checkForParameterPackages(self, userOps, packagesPath):
		if userOps.has_key('PARAMETER_PACKAGE') and userOps['PARAMETER_PACKAGE'] != None:
			if type(userOps['PARAMETER_PACKAGE']) == str: packages = [userOps['PARAMETER_PACKAGE']]
			else: packages = userOps['PARAMETER_PACKAGE']
			for package in packages:
				pPackageFile = os.path.join(packagesPath, package+'.py')
				if not os.path.exists(pPackageFile):
					utilities.error('parameter package', 'cannot find parameter package file '+pPackageFile+"!!!")
				packageOps = file2optionsDict(pPackageFile)
				overwrites = []
				for key, val in packageOps.iteritems():
					if self.userOps.has_key(key) and self.userOps[key] != val: overwrites.append(key)
					self.userOps[key] = val
				if self.v >= 3: print "\tUsing parameter package",  os.path.join('packages', package+'.py'), "("+str(len(packageOps))+" parameters)."
				if len(overwrites) > 0: print "\t\toverwrote "+str(len(overwrites))+" existing options -> "+', '.join(overwrites)
		return userOps
	############################################################################
	def __getattr__(self, name): # for ops.OPTION_NAME calls...
		if self.userOps.has_key(name): return self.userOps[name]
		elif self.defaultOps.has_key(name): return self.defaultOps[name]
		else: return self.userOps[name]
		#else: # die
		#	utilities.error("ops", "AudioGuide needs an option called "+name+" but your options file does not have it!")
	############################################################################
	def __setattr__(self, name, val): # so we can keep track of changed options, know what needs to be redone.
		if self.__dict__.has_key(name) and self.__dict__[name] == val: pass # not changed
		self.__dict__[name] = val # normal operation
		if name in ['PRINT_LENGTH', 'VERBOSITY', 'TARGET_PATH', 'SOURCE_PATH', 'OPTIONS_DIR', 'TMP_PATH', 'CSOUND_MAKE_SCORE', 'CSOUND_SCORE_PATH', 'MAKE_QLIST', 'OUTPUT_QLIST_NAME', 'MAKE_OPEN_MUSIC_SCORE', 'OM_SCORE_NAME', 'MAKE_MIDIFILE', 'MIDI_SCORE_NAME', 'MIDI_FILE_NAME', 'SAVE_TARGET_SEGMENT_LABELS', 'TARGET_SEGMENT_LABELS', 'TARGET_SEGMENT_MULTI_LABELS', 'SAVE_CONFIG_FILE', 'CONFIG_FILE_NAME', 'SAVE_TARGET_SEGMENT_OR_LINK', 'TARGET_SEGMENT_OR_LINK', 'MAKE_OUTPUT_MIX', 'CSOUND_OUTPUT_MIX_FILE', 'NEVER_STORE_IN_DATABASE']: self.needToInitalize = True ; self.needToLoadTgt = True ; self.needToLoadSrc = True ; self.needToNormalize = True
		if name in ['TARGET', 'TARGET_FADE_IN_SEC', 'TARGET_FADE_OUT_SEC', 'TARGET_ONSET_THRESHOLD', 'TARGET_ONSET_DESCRIPTORS', 'MAX_SIMULTANEOUS_SAMPLES', 'TARGET_ONSET_THRESHOLD_SIM_PENALITY', 'MAX_SEGMENTS_PER_ONSET', 'TARGET_CORRECT_ODF_LATENCY', 'ODF_LATENCY_FRAMES', 'ODF_NUMB_AVG', 'TARGET_SEGMENT_DROP_RATIO', 'TARGET_OFFSET_DB_ABOVE_MIN', 'TARGET_SEGMENT_LENGTH_DROP_METHOD', 'TARGET_OFFSET_RISE_RATIO', 'TARGET_ONSET_ENVELOPE', 'TARGET_OFFSET_ENVELOPE', 'TARGET_MIN_SEGMENT_LENGTH', 'TARGET_MAX_SEGMENT_LENGTH', 'TARGET_INCREMENT', 'TARGET_INCREMENT_ENVELOPE', 'TARGET_MIN_INCREMENT', 'TARGET_SCALE_AMP']: self.needToLoadTgt = True ; self.needToNormalize = True
		if name in ['CORPUS', 'LIMIT_CORPUS_BY_DESCRIPTOR', 'CAN_USE_SAME_SEGMENT_TWICE', 'CAN_REUSE_SEGMENT_AFTER', 'CORPUS_RESTRICT_DISCRIPTOR_IN_TIME', 'CORPUS_RESTRICT_NUMBER_STREAMS', 'VOICE_RESTRICT_ENTRANCES_IN_TIME', 'MAX_ACTIVE_VOICES', 'ROTATE_VOICES', 'SOUNDFILE_EXTENSIONS', 'SPECTRAL_POWER_THRESHOLD']: self.needToLoadSrc = True ; self.needToNormalize = True
		if name in ['NORMALIZE_METHOD', 'NORMALIZE_DEFAULT_METHOD', 'NORMALIZE_SCALE_STD_DEV', 'NORMALIZE_TARGET_SEGMENTS_ONLY']: self.needToNormalize = True
		
		if name in ['DESCRIPTORS', 'FIRST_PASS_DESCRIPTORS', 'FIRST_PASS_PERCENT']:
			self.needToRemakeAnalLists = True
			self.needToNormalize = True
	
	############################################################################
	def writeConfigurationFile(self, filename):
		neverWrite = []
		orderedOps = readOptionOrder(self.userOps['DEFAULT_PATH'])
		toWrite = []
		nothingWritten = True
		skipWrite = ['AG_PATH', 'SCRIPT_PATH', 'OPS_PATH', 'DEFAULT_PATH', 'PARAMETER_PACKAGE']
		
		addUserOps = [] # for extra user ops that are not in defaults..  
		for name, val in self.userOps.iteritems():
			if name not in skipWrite and not self.defaultOps.has_key(name): # items that are in userops but not defaults
				addUserOps.append(name)
		for i, op in enumerate(addUserOps): # add unsupplied user ops to the write list
			orderedOps[0].insert(i+1, op)
		
		for array in orderedOps:
			sectionID = array[0]
			sectionWritten = False
			for name in array[1:]:
				if self.userOps.has_key(name) and (not self.defaultOps.has_key(name) or self.userOps[name] != self.defaultOps[name]):
					if not sectionWritten:
						sectionString = makeSectionName(sectionID)
						if not nothingWritten: sectionString = "\n"+sectionString
						toWrite.append(sectionString)
						sectionWritten = True
						nothingWritten = False
					toWrite.append(name+' = '+self.filterSpecialNames(name))
		configFile = open(filename, 'w')
		for line in toWrite:
			configFile.write(line+'\n')
		configFile.close()
	############################################################################
	def filterSpecialNames(self, name):
		if name in ['DISTANCE_DESCRIPTORS', 'FIRST_PASS_DESCRIPTORS']:
			tmp = {}
			for key, val in self.userOps[name].iteritems():
				if type(val) == tuple:
					if val[0] == 0: continue
				else: 
					if val == 0: continue
				tmp[key] = val
			string = str(tmp)
		else:
			if type(self.userOps[name]) == str:
				string = '"'+str(self.userOps[name])+'"'
			else:
				string = str(self.userOps[name])
			
		return string

#          ,)(8)).
#        (()))())()).
#       (()"````"::= )
#       )| _    _ ::= )
#      (()(o)/ (o) ?(/)
#       )(  c    ( :(/) --> WTF?
#      (( \ .__. ;,/(/)
#        ) `.___,'/ (/)
#           |    | (/)
#         _.'    ,\(/)__
#     _.-"   ` u    (/) ".
#   ,"               ^    \
#  /                      |

def makeSectionName(sectionID):
	lenP = (56-len(sectionID))/2
	return "#"*lenP+sectionID+"#"*lenP

	
def readOptionOrder(file):
	fh = open(file, 'r')
	lines = fh.readlines()
	fh.close()
	orderedOps = []
	sectionOps = ['  REQUIRED  '] # first section is REQUIRED
	sectionCounter = 0
	for line in lines:
		line = line.replace('\n','')
		if line == '': continue
		if line[0:3] == '###':
			sectionCounter += 1
			orderedOps.append(sectionOps)
			sectionOps = [line.replace('#','')]
		elif line[0] == '#': continue
		else:
			sectionOps.append(line.split()[0])
	orderedOps.append(sectionOps) # append last section
	return orderedOps


def file2optionsDict(file):
	# loads file into a dict
	dict = {}
	fh = open(file, 'r')
	lines = fh.read()
	fh.close()
	try:
		exec(lines, globals(), dict)
	except:
		utilities.error("ops", "there is a python syntax error in your options file\n\n\t"+str(sys.exc_info()[1]))
	return dict

def string2optionsDict(string):
	# loads file into a dict
	dict = {}
	try:
		exec(string, globals(), dict)
	except:
		utilities.error("ops", "there is a python syntax error in your options string\n\n\t"+str(sys.exc_info()[1]))
	return dict


# CHECK USER VARS AND MAKE SURE THAT THEY ARE THE CORRECT TYPE
opsCast = {
#'TARGET': {'type': [list], 'len': 3},
'CORPUS': {'type': [list]},
'CSOUND_MAKE_SCORE': {'type': [bool]},
'CSOUND_SCORE_PATH': {'type': [str]},
'SIMULTANEOUS_SELECTION_METHOD': {'among': [None, "none", "None", "mixture"]},
#'TARGET_ONSET_THRESHOLD': {'between': (-1000, 0)}, -- sometimes in over 1 if a different odf is used!
################  GLOBAL VARIABLES  ################


'NEVER_STORE_IN_DATABASE': {'type': [list]},
############ TARGET  ############
'TARGET_DESCRIPTORS_MOD': {'type': [list]},





}

def checkOptionCasting(opsDict):
	for key, val in opsDict.iteritems():
		if opsCast.has_key(key):
			for castKey, castVal in opsCast[key].iteritems():
				if castKey == "type":
					if type(val) not in castVal:
						utilities.error('ops', 'option '+str(key)+' must be among the datatypes ->'+str(castVal))
				elif castKey == "len":
					if len(val) != castVal:
						utilities.error('ops', 'option '+str(key)+' must be '+str(castVal)+' items long')
				elif castKey == "among":
					if val not in castVal:
						utilities.error('ops', 'option '+str(key)+' must be one of the following choices '+str(castVal))
				elif castKey == "between":
					if type(val) not in [int, float] or (val < castVal[0] or val > castVal[1]):
						utilities.error('ops', 'option '+str(key)+' must be a number between '+str(castVal[0])+' and '+str(castVal[1]))

