#!/usr/bin/env python2.7
import sys, os, platform
import numpy as np
try:
	import json as json
except ImportError:
	try:
		import simplejson as json
	except: pass

ag_dir = sys.argv[1]
config_file = sys.argv[2]
output_file_checksum = sys.argv[3]



def clusterAnalysis(clusteringMatrix, numb_clusters):# cluster analysis		
	Y = hcluster.pdist(clusteringMatrix)
	Z = hcluster.linkage(Y, method='average')
	clusterAssignment = hcluster.fcluster(Z, numb_clusters, criterion='maxclust') 
	clusterAssignment -= 1
	# get cluster centroids
	# Sum the vectors in each cluster
	lens = {}      # will contain the lengths for each cluster
	centroids = {} # will contain the centroids of each cluster
	for idx, clno in enumerate(clusterAssignment):
		centroids.setdefault(clno, np.zeros(clusteringMatrix.shape[1])) 
		centroids[clno] += clusteringMatrix[idx,:]
		lens.setdefault(clno,0)
		lens[clno] += 1
	# Divide by number of observations in each cluster to get the centroid
	for clno in centroids:
		centroids[clno] /= float(lens[clno])
	return clusterAssignment, centroids, lens



class BreakIt(Exception): pass
## ^ used for double-loop exiting
def mapClusterNodes(targetCentroids, corpusCentroids, numberClusters):
	#################
	#  BY DISTANCE	#
	#################
	minimumFoundError = (sys.maxint, None) # errorAmount, permSeriesIdx
	permutations = audioguide.utilities.permutations(range(numberClusters))

	calculatedDistances = {} # calculate pair-wise array distance only when not already done!

	for idx, perm in enumerate(permutations):
		thisError = 0.
		for t, c in enumerate(perm):
			try:
				if not calculatedDistances.has_key((t, c)):
					calculatedDistances[(t, c)] = np.sum(np.power(targetCentroids[t]-corpusCentroids[c], 2))
					print t, c, calculatedDistances[(t, c)]
				thisError += calculatedDistances[(t, c)]
				if thisError > minimumFoundError[0]: raise BreakIt
			except BreakIt: pass
		if thisError < minimumFoundError[0]:
			minimumFoundError = (thisError, perm)
			print "NEW MIN", minimumFoundError
		#print thisError
		#print idx

	tgtToCpsMap = {}
	for i in range(numberClusters):
		tgtToCpsMap[minimumFoundError[1][i]] = i
	return tgtToCpsMap






def writeSdifDictFiles(sdifDict):	
	for sffile, values in sdifDict.iteritems():
		#print list(values)
		sdiffile = "%s-cluster-%s.sdif"%(SdifInterface.rawData[sffile]['sdiffileroot'], output_file_checksum)
		# write a SdifFile
		sdif = pysdif.SdifFile(sdiffile, 'w')
		# define new frame and matrix types
		sdifvals = ['clusterid']
		for didx in range(values.shape[1]):
			sdifvals.append('cluster_cent%i'%didx)
		
		sdif.add_frame_type('1CST', '1CST clustermatrix')
		sdif.add_matrix_type('1CST', ', '.join(sdifvals))
		# now you can begin adding frames
		for f in xrange(len(values)):
			frame = sdif.new_frame('1CST', f*frameSec)
			frame.add_matrix('1CST', values[f] )
			#print f*frameSec, values[f]
			frame.write()
		sdif.close()
		print "\t\tWROTE", os.path.split(sdiffile)[1]	






tmpconfig = json.load(open(config_file, 'r'))
config = {} # get rid of unicode!!!
for k, v in tmpconfig.iteritems():
	if k in ['AG_PATH', 'DEFAULT_PATH', 'OPS_PATH', 'SCRIPT_PATH']: continue
	if type(v) is unicode: v = str(v)
	config[str(k)] = v


config['VERBOSITY'] = 0
config['OUTPUT_LOG_FILE'] = None
config['SAVE_CONFIG_FILE'] = False
config['SAVE_TARGET_SEGMENT_LABELS'] = False
config['PARAMETER_PACKAGE'] = None
config['FIRST_PASS_DESCRIPTORS'] = {}
config['FIRST_PASS_PERCENT'] = 0




# import audioguide submodules
sys.path.append( os.path.join(ag_dir, '..') ) # audioguide import
import audioguide
sys.path.append( audioguide.config['py_compiled_lib_dir'] )

#os.path.join(ag_dir, '..')



import pysdif
import hcluster

main = audioguide.main(ag_dir, 'defaults.py', pythonicUserOptions=config)
main.loadDatabase()
main.loadTarget()
main.normalizeData()


#  just for convenience...
descriptorData = audioguide.descriptorData
tgt = audioguide.tgt
cps_handles = audioguide.ag.sourceHandles
descriptorData = audioguide.descriptorData
SdifInterface = audioguide.SdifInterface
frameSec = audioguide.ops.DESCRIPTOR_HOP_SIZE_SEC
numb_clusters = audioguide.ops.DESCRIPTOR_NUMB_CLUSTS

sdifDict = {}
descriptors = []
weights = []
for d in descriptorData.target_load:
	if d.weight == 0: continue
	descriptors.append(d)
	weights.append( d.weight )


#print weights
#sys.exit()

########################
## do them separately ##
########################
###############
## do target ##
###############
descriptor_total_length = 0
for st, ed in tgt.segFrames: descriptor_total_length += ed-st
clusteringMatrix = np.empty((descriptor_total_length, len(descriptors)))
f_cnt = 0
for st, ed in tgt.segFrames:
	for f in xrange(ed-st):
		for didx, d in enumerate(descriptors):
			clusteringMatrix[f_cnt+f][didx] = tgt.data['norm'][d.name][st+f]
			#print d.name, clusteringMatrix[f_cnt+f][didx]
	f_cnt += ed-st

clusteringMatrix *= np.array(weights)
targetClusterAssignment, targetCentroids, lens = clusterAnalysis(clusteringMatrix, numb_clusters)
sdifDict[tgt.filename] = np.zeros((int(tgt.sfdur/frameSec), len(descriptors)+1))



###############
## do corpus ##
###############
descriptor_total_length = 0
for h in cps_handles: descriptor_total_length += h.data['raw']['effDur-seg'][0]

# make emptry matrix
clusteringMatrix = np.empty((descriptor_total_length, len(descriptors)))
# fill it!
f_cnt = 0
for h in cps_handles:
	for f in xrange(h.data['raw']['effDur-seg'][0]):
		for didx, d in enumerate(descriptors):
			clusteringMatrix[f_cnt+f][didx] = h.data['norm'][d.name][f]
			#print h.name, h.startSec, d.name, clusteringMatrix[f_cnt+f][didx]
	f_cnt += h.data['raw']['effDur-seg'][0]

clusteringMatrix *= np.array(weights)
corpusClusterAssignment, corpusCentroids, lens = clusterAnalysis(clusteringMatrix, numb_clusters)



tgtToCpsMap = mapClusterNodes(targetCentroids, corpusCentroids, numb_clusters)
print tgtToCpsMap
print corpusClusterAssignment[:10]

for idx in range(len(corpusClusterAssignment)):
	corpusClusterAssignment[idx] = tgtToCpsMap[corpusClusterAssignment[idx]]


f_cnt = 0
for st, ed in tgt.segFrames:
	for f in xrange(ed-st):
		for didx, d in enumerate(descriptors):
			sdifDict[tgt.filename][st+f][0] = targetClusterAssignment[f_cnt+f]
			sdifDict[tgt.filename][st+f][1:] = targetCentroids[targetClusterAssignment[f_cnt+f]]
	f_cnt += ed-st


f_cnt = 0
for h in cps_handles:
	if not sdifDict.has_key(h.filename):
		lengthFrames = int(h.sfdur/frameSec)
		sdifDict[h.filename] = np.zeros((lengthFrames, len(descriptors)+1))
	startframe = int(h.startSec/frameSec)
	for f in xrange(h.data['raw']['effDur-seg'][0]):
		sdifDict[h.filename][startframe+f][0] = corpusClusterAssignment[f_cnt+f]
		sdifDict[h.filename][startframe+f][1:] = corpusCentroids[corpusClusterAssignment[f_cnt+f]]
	f_cnt += h.data['raw']['effDur-seg'][0]
# now loop through files and create an sdif for each




writeSdifDictFiles(sdifDict)







#
#
#
#
#
#
#sys.exit(0)
#
#
## find length of all time varying metrics
#descriptor_total_length = 0
#for st, ed in tgt.segFrames: descriptor_total_length += ed-st
#for h in cps_handles: descriptor_total_length += h.data['raw']['effDur-seg'][0]
#
#
## make emptry matrix
#clusteringMatrix = np.empty((descriptor_total_length, len(descriptors)))
## fill it!
#f_cnt = 0
#for st, ed in tgt.segFrames:
#	for f in xrange(ed-st):
#		for didx, d in enumerate(descriptors):
#			clusteringMatrix[f_cnt+f][didx] = tgt.data['norm'][d.name][st+f]
#			#print d.name, clusteringMatrix[f_cnt+f][didx]
#	f_cnt += ed-st
#for h in cps_handles:
#	for f in xrange(h.data['raw']['effDur-seg'][0]):
#		for didx, d in enumerate(descriptors):
#			clusteringMatrix[f_cnt+f][didx] = h.data['norm'][d.name][f]
#			#print h.name, h.startSec, d.name, clusteringMatrix[f_cnt+f][didx]
#	f_cnt += h.data['raw']['effDur-seg'][0]
#
#
#clusteringMatrix *= np.array(weights)
#
#clusterAssignment, centroids, lens = clusterAnalysis(clusteringMatrix, numb_clusters)
#
#
## start counting from 1, 0 = no cluster found!
##		# put cluster assignments into a dict with file name and time as key.
#clusterDict = {}
#lengthFrames = int(tgt.sfdur/frameSec)
#clusterDict[tgt.filename] = np.zeros(lengthFrames)
#
#
#
#f_cnt = 0
#for st, ed in tgt.segFrames:
#	for f in xrange(ed-st):
#		clusterDict[tgt.filename][st+f] = clusterAssignment[f_cnt+f]
#	print clusterAssignment[f_cnt:f_cnt+(ed-st)]
#	f_cnt += ed-st
#for h in cps_handles:
#	if not clusterDict.has_key(h.filename):
#		lengthFrames = int(h.sfdur/frameSec)
#		clusterDict[h.filename] = np.zeros(lengthFrames)
#	startframe = int(h.startSec/frameSec)
#	for f in xrange(h.data['raw']['effDur-seg'][0]):
#		clusterDict[h.filename][startframe+f] = clusterAssignment[f_cnt+f]
#	f_cnt += h.data['raw']['effDur-seg'][0]
## now loop through files and create an sdif for each
#for sffile in clusterDict:
#	sdiffile = "%s-cluster-%s.sdif"%(SdifInterface.rawData[sffile]['sdiffileroot'], output_file_checksum)
#	# write a SdifFile
#	sdif = pysdif.SdifFile(sdiffile, 'w')
#	# define new frame and matrix types
#	sdif.add_frame_type('1CST', '1CST clusterid')
#	sdif.add_matrix_type('1CST', 'Column1')
#	# now you can begin adding frames
#	for f in xrange(len(clusterDict[sffile])):
#		frame = sdif.new_frame('1CST', f*frameSec)
#		frame.add_matrix('1CST', clusterDict[sffile][f:f+1] )
#		frame.write()
#	sdif.close()
#	print "WROTE", sdiffile	
#sys.exit(0)
#
#
#
#
#
#
