import sys, os, subprocess
sys.path.insert(0, os.path.dirname(__file__)) # look here first
import pysdif
from numpy import zeros
from utilities import error, popen_execute_command, executeCommand, checkIfFileIsNewer

try:
	import json as json
except ImportError:
	try:
		import simplejson as json
	except: pass


class SdifInterface:
	def __init__(self, ops, path, ircamdescriptor_bin, pm2_bin, supervp_bin):	
		self.ops = ops
		self.path = path
		self.ircamdescriptor_bin = ircamdescriptor_bin
		self.pm2_bin = pm2_bin
		self.supervp_bin = supervp_bin
		# anal
		self.winLengthSec = ops.DESCRIPTOR_WIN_SIZE_SEC
		self.hopLengthSec = ops.DESCRIPTOR_HOP_SIZE_SEC
		self.resampleRate = ops.DESCRIPTOR_RESAMPLE_RATE
		self.windowType = ops.DESCRIPTOR_WINDOW_TYPE
		self.numbBands = ops.DESCRIPTOR_NUMB_BANDS
		self.F0MaxAnalysisFreq = ops.DESCRIPTOR_F0_MAX_ANALYSIS_FREQ
		self.F0MinFrequency = ops.DESCRIPTOR_F0_MIN_FREQUENCY
		self.F0MaxFrequency = ops.DESCRIPTOR_F0_MAX_FREQUENCY
		self.F0AmpThreshold = ops.DESCRIPTOR_F0_AMP_THRESHOLD
		self.F0Quality = ops.DESCRIPTOR_F0_QUALITY
		self.forceAnal = ops.DESCRIPTOR_FORCE_ANALYSIS
		# PEAKS SUPERVP ANALYSIS
		self.peaksThreshold = 70
		self.numbPeaks = ops.DESCRIPTOR_NUMB_PEAKS
		# PARTIALS PM2
		self.partialThreshold = 70
		self.maxpartials = 12
		self.minParConnectSec = 0.317000001
		self.minParConnectFrqRatio = 1.28920708
		self.minParDuration = 0.0
		self.agDescriptToSdif = agDescriptToSdif
		# CLUSTER ANALYSIS
		self.numbClust = ops.DESCRIPTOR_NUMB_CLUSTS
		self.clustDescriptDict = ops.DESCRIPTOR_CLUSTS_DESCRIPTOR_DIM

		
		# mfccs
		for b in range(self.numbBands):
			self.agDescriptToSdif['mfcc%i'%b] = ('descriptors', '1DSC', '1MFC', b, 0)
		# peaks
		for p in range(self.numbPeaks):
			self.agDescriptToSdif['peakfrq%i'%p] = ('peaks', '1PIC', '1PIC', p, 0)
			self.agDescriptToSdif['peakamp%i'%p] = ('peaks', '1PIC', '1PIC', p, 1)
		# partials
		for p in range(self.maxpartials):
			self.agDescriptToSdif['partialfrq%i'%p] = ('partials', '1TRC', '1TRC', p, 1)
			self.agDescriptToSdif['partialamp%i'%p] = ('partials', '1TRC', '1TRC', p, 2)
		# cluster centroids
		for p in range(len(self.clustDescriptDict)):
			self.agDescriptToSdif['cluster_cent%i'%p] = ('cluster', '1CST', '1CST', 0, p+1)



		self.rawData = {} # file, descriptdict
		self.amplitudedescriptors = ['power', 'rms']
		descriptors = ['SignalZeroCrossingRate', 'TotalEnergy', 'SpectralCentroid', 'SpectralSpread', 'SpectralSkewness', 'SpectralKurtosis', 'SpectralSlope', 'SpectralDecrease', 'SpectralRolloff', 'SpectralVariation', 'Loudness', 'RelativeSpecificLoudness', 'Spread', 'Sharpness', 'PerceptualSpectralCentroid', 'PerceptualSpectralSpread', 'PerceptualSpectralSkewness', 'PerceptualSpectralKurtosis', 'PerceptualSpectralSlope', 'PerceptualSpectralDecrease', 'PerceptualSpectralRolloff', 'PerceptualSpectralVariation', 'PerceptualSpectralDeviation', 'PerceptualOddToEvenRatio', 'PerceptualTristimulus', 'MFCC', 'SpectralFlatness', 'SpectralCrest', 'FundamentalFrequency', 'NoiseEnergy', 'Noisiness', 'Inharmonicity', 'HarmonicEnergy', 'HarmonicSpectralCentroid', 'HarmonicSpectralSpread', 'HarmonicSpectralSkewness', 'HarmonicSpectralKurtosis', 'HarmonicSpectralSlope', 'HarmonicSpectralDecrease', 'HarmonicSpectralRolloff', 'HarmonicSpectralVariation', 'HarmonicSpectralDeviation', 'HarmonicOddToEvenRatio', 'HarmonicTristimulus', 'Chroma']
		energydescriptors = ['TemporalIncrease', 'TemporalDecrease', 'TemporalCentroid', 'EffectiveDuration', 'LogAttackTime', 'AmplitudeModulation', 'EnergyEnvelope']
		dlist = ';~~~~~~~~~~~~~~~~~~~descriptors~~~~~~~~~~~~~~~~~~~~~\n'
		for d in descriptors: dlist += d+'  = ShortTime\n'
		edlist = ';~~~~~~~~~~~~~~~~energy descriptors~~~~~~~~~~~~~~~~\n'
		for d in energydescriptors: edlist += d+'  = 1\n'
		### setup ircam binary config text file	
		self.config_loc = '/tmp/ircam.config'
		self.config_text = '''[Parameters]
ResampleTo = %i
SaveShortTermTMFeatures = 1
WindowType = %s
[StandardDescriptors]
; in seconds
WindowSize = %f
HopSize = %f
TextureWindowsFrames = -1
TextureWindowsHopFrames = -1		
PerceptualBands = %i
ReducedBands = 4
MFCCs = %i
AutoCorrelationCoeffs = 12
Harmonics = 20
DeviationStopBand = 10
RolloffThreshold = 0.95
DecreaseThreshold = 0.4
NoiseThreshold = 0.15
F0MaxAnalysisFreq = %i
F0MinFrequency = %i
F0MaxFrequency = %i
F0AmpThreshold = %i
F0Quality = %f

%s

[EnergyDescriptors]

WindowSize = 0.1
HopSize = 0.025
TextureWindowsFrames = -1
TextureWindowsHopFrames = -1

%s'''%(self.resampleRate, self.windowType, self.winLengthSec, self.hopLengthSec, self.numbBands, self.numbBands, self.F0MaxAnalysisFreq, self.F0MinFrequency, self.F0MaxFrequency, self.F0AmpThreshold, self.F0Quality, dlist, edlist)
	########################################################
	########################################################
	def checkSdifResource(self, handle, sffile):
		sffile = os.path.abspath(sffile)
		if not self.rawData.has_key(sffile):
			self.initSdifResource(sffile)
			self.rawData[sffile]['handle'] = []
		self.rawData[sffile]['handle'].append(handle)
		return self.rawData[sffile]['info']['lengthsec'], self.rawData[sffile]['info']['channels']
	########################################################
	########################################################
	def getSdifFileName(self, filetype, sffile, manualCheck=None):
		if manualCheck == None:
			return self.rawData[sffile]['sdiffileroot']+'-'+filetype+'-'+self.rawData[sffile]['checksum']+'.sdif'
		else: 
			return self.rawData[sffile]['sdiffileroot']+'-'+filetype+'-'+manualCheck+'.sdif'
	########################################################
	########################################################
	def initSdifResource(self, sffile):
		'''if analysis files don't exist, this function automatically makes
		the ircamdescriptor SDIF analysis because it is required for all
		sound files used.  then checks to see if we need other sdif files
		made -- such as peaks with supervp or partials with pm2.'''
		self.rawData[sffile] = {}
		sfroot, sfhead = os.path.split(sffile)
		sfheadroot, sfheadext = os.path.splitext(sfhead)
		sdifdir = os.path.join(os.path.dirname(__file__), 'data_sdif')
		jsondir = os.path.join(os.path.dirname(__file__), 'data_json')
		if not os.path.exists(sdifdir): os.makedirs(sdifdir)
		if not os.path.exists(jsondir): os.makedirs(jsondir)
		self.rawData[sffile]['sdiffileroot'] = os.path.join(sdifdir, sfheadroot)
		self.rawData[sffile]['jsonfileroot'] = os.path.join(jsondir, sfheadroot)
		self.rawData[sffile]['checksum'] = listToCheckSum([sffile, self.resampleRate, self.windowType, self.numbBands, self.numbBands, self.winLengthSec, self.hopLengthSec, self.F0MaxAnalysisFreq, self.F0MinFrequency, self.F0MaxFrequency, self.F0AmpThreshold, self.F0Quality])

		descriptorfile = self.getSdifFileName('descriptors', sffile)
		infofile = '%s-%s.json'%(self.rawData[sffile]['jsonfileroot'], listToCheckSum([sffile]))
		
		self.rawData[sffile]['info'] = {}
		if os.path.exists(infofile) and os.path.exists(descriptorfile) and not checkIfFileIsNewer(sffile, infofile) and not self.forceAnal:
			self.rawData[sffile]['info'].update(json.load(open(infofile, 'r')))
		else:	# then we need to run ircam descriptor analysis
			# write tmp config files
			fh = open(self.config_loc, 'w')
			fh.write(self.config_text)
			fh.close()
			self.rawData[sffile]['info'] = popen_execute_command([self.ircamdescriptor_bin, sffile, self.config_loc, '-o'+descriptorfile], stdoutReturnDict={('sr', 0): ('sr', 2, int), ('samples', 0): ('lengthsamples', 2, int), ('channel(s):', 0): ('channels', 1, int)})
			self.rawData[sffile]['info']['lengthsec'] = self.rawData[sffile]['info']['lengthsamples']/float(self.rawData[sffile]['info']['sr'])
			jh = open(infofile, 'w')
			json.dump(self.rawData[sffile]['info'], jh)
			jh.close()
		self.rawData[sffile]['arraylen'] = int(self.rawData[sffile]['info']['lengthsec']/self.hopLengthSec) + 1
		self.rawData[sffile]['loaded'] = False
	########################################################
	########################################################
	def PartialAnalysis(self, sffile):
		analfile = self.getSdifFileName('partials', sffile)
		if not os.path.exists(analfile) and not self.forceAnal:
			windowsize = self.winLengthSec*self.rawData[sffile]['info']['sr']
			oversamp = self.winLengthSec/self.hopLengthSec
			command = '%s --quiet -t -S%s -Apar  -N%i -M%i --oversamp=%i -Whamming  -OS -p1 --mode=0 -q%i -m%i -a0 -r0 -Ct%f -Cf%f --devFR=0.059463143 --devFC=0 --devA=0.5 --devM=1 --devK=3 -L%f %s'%(self.pm2_bin, sffile, windowsize, windowsize, oversamp, self.maxpartials, self.partialThreshold, self.minParConnectSec, self.minParConnectFrqRatio, self.minParDuration, analfile)
			print
			print
			print command
			print
			print 
			popen_execute_command(command.split()) 
	########################################################
	########################################################
	def PeakAnalysis(self, sffile):
		analfile = self.getSdifFileName('peaks', sffile)
		if not os.path.exists(analfile) and not self.forceAnal:
			windowsize = self.winLengthSec*self.rawData[sffile]['info']['sr']
			oversamp = self.winLengthSec/self.hopLengthSec
			popen_execute_command([self.supervp_bin, '-quiet', '-t', '-ns', '-U', '-S'+sffile, '-Apic', '25', 'n%i'%self.numbPeaks, '-Np0', '-M%fs'%self.winLengthSec, '-oversamp', '%i'%oversamp, '-Whamming', '-OS0', analfile], exitOnError=False) 
			# supervp is fucked up and streams A LOT of data to stderr.  why would those mofos do something like that?
	########################################################
	########################################################
	def ClusterIdAnalysis(self, sffile):
		# alter target and corpus entries so that they include full paths.
		allOps = {}
		outputFile = '/tmp/ag-sdifanal.config.json'
		# make full paths for corpus and target entries so that we find the right file on an outboard analysis
		null, tgtPath, null = self.path.test(self.ops.userOps['TARGET'][0], self.ops.TARGET_PATH)
		self.ops.userOps['TARGET'][0] = tgtPath
		for idx, item in enumerate(self.ops.userOps['CORPUS']):
			if type(item) == dict:
				name = item['name']
				null, cpsPath, null = self.path.test(name, self.ops.SOURCE_PATH)
				self.ops.userOps['CORPUS'][idx]['name'] = cpsPath
			else:
				null, cpsPath, null = self.path.test(item, self.ops.SOURCE_PATH)
				self.ops.userOps['CORPUS'][idx] = cpsPath
		allOps.update(self.ops.userOps)
		allOps['DESCRIPTORS'] = self.ops.DESCRIPTOR_CLUSTS_DESCRIPTOR_DIM
		checksumlist = []
		for k, v in allOps.iteritems():
			checksumlist.extend((k, str(v)))
		manual_checksum = listToCheckSum([checksumlist])

		analfile = self.getSdifFileName('cluster', sffile, manualCheck=manual_checksum)
		if os.path.exists(analfile) and not self.forceAnal: return analfile
		print "\n\nMAKING A CLUSTER-ID SDIF ANALYSIS"
		print "\tTARGET: %s"%tgtPath
		print "\tCORPUS: %s"%self.ops.userOps['CORPUS']
		print "\tCLUSTER DIMENSIONS: %s"%self.ops.DESCRIPTOR_CLUSTS_DESCRIPTOR_DIM
		print "\tNUMBER OF CLUSTERS: %s"%self.ops.DESCRIPTOR_NUMB_CLUSTS
		allOps['DESCRIPTORS'] = self.ops.DESCRIPTOR_CLUSTS_DESCRIPTOR_DIM
		# write config
		configFile = '/tmp/ag-sdifanal.config.json'
		fh = open(configFile, 'w')
		json.dump(allOps, fh)
		fh.close()
		print '%s %s %s %s'%(os.path.join(self.ops.userOps['AG_PATH'], 'scripts', 'make_clusterid_sdif.py'), self.ops.userOps['AG_PATH'], configFile, manual_checksum)
		print "\n\n"
		executeCommand('%s %s %s %s'%(os.path.join(self.ops.userOps['AG_PATH'], 'scripts', 'make_clusterid_sdif.py'), self.ops.userOps['AG_PATH'], configFile, manual_checksum))
		return analfile
	########################################################
	########################################################
	def loadDescriptorSdifData(self, sffile, descriptList):
		sffile = os.path.abspath(sffile)
		self.rawData[sffile]['neededfiles'] = {}
		self.rawData[sffile].update({ 'load': {}, 'load_cnt': {} })
		for d in descriptList:	
			# find out which files to query
			filetype, frame, matrix, row, col = agDescriptToSdif[d.name]
			if filetype not in self.rawData[sffile]['neededfiles']: self.rawData[sffile]['neededfiles'][filetype] = {}
			self.rawData[sffile]['neededfiles'][filetype][d.name] = frame, matrix, row, col
			# make data space
			self.rawData[sffile]['load'][d.name] = zeros(self.rawData[sffile]['arraylen'])
			self.rawData[sffile]['load_cnt'][d.name] = 0
		# done alloc memory	
		# make peak of partials analysis if neede byt the descriptrs that the user wants
		customNames = {}
		if self.rawData[sffile]['neededfiles'].has_key('partials'): # need pm2 analysis!
			self.PartialAnalysis(sffile)
		if self.rawData[sffile]['neededfiles'].has_key('peaks'): # need supervp analysis!
			self.PeakAnalysis(sffile)
		if self.rawData[sffile]['neededfiles'].has_key('cluster'): # need supervp analysis!
			customnamed = self.ClusterIdAnalysis(sffile)
			customNames['cluster'] = customnamed



		# loop over needed files
		for filetype in self.rawData[sffile]['neededfiles']:
			reqFrameTypes = []
			frameMatrixDict = {}
			for name, (frame, matrix, row, col) in self.rawData[sffile]['neededfiles'][filetype].iteritems():
				if frame not in reqFrameTypes: reqFrameTypes.append(frame)
				if not frameMatrixDict.has_key((frame, matrix)):
					frameMatrixDict[(frame, matrix)] = []
				frameMatrixDict[(frame, matrix)].append((name, row, col))
			#print "READ", filetype, frameMatrixDict, reqFrameTypes
			if customNames.has_key(filetype):
				file = customNames[filetype]
			else:
				file = self.getSdifFileName(filetype, sffile)
			if not os.path.exists(file):
				print "CANNOT FIND", file
				sys.exit()
			sdfh = pysdif.SdifFile(file)
			for f in sdfh:
				#print f.signature
				if f.signature not in reqFrameTypes: continue
				for m in f:
					#print m.signature
					if not frameMatrixDict.has_key((f.signature, m.signature)): continue
					for name, col, row in frameMatrixDict[(f.signature, m.signature)]:
						#print list( m.get_data(copy=False) )
						if self.rawData[sffile]['load_cnt'][name] >= self.rawData[sffile]['arraylen']:
							continue
						try:
							self.rawData[sffile]['load'][name][self.rawData[sffile]['load_cnt'][name]] = m.get_data(copy=False)[col][row]
						except IndexError: # data not there, maybe asking for peaks that are not found?
							self.rawData[sffile]['load'][name][self.rawData[sffile]['load_cnt'][name]] = 0
						self.rawData[sffile]['load_cnt'][name] += 1		
			# close this sdif file handle
			sdfh.close()
		self.rawData[sffile]['loaded'] = True
	########################################################
	########################################################
	def getDescriptorSegment(self, sffile, startsec, endsec, descriptList, dataDest, envelopeMask):
		sffile = os.path.abspath(sffile)
		# load data -- only happens on the first call
		if not self.rawData[sffile]['loaded']: self.loadDescriptorSdifData(sffile, descriptList)
		durationsec = endsec-startsec
		startframe = int(startsec/self.hopLengthSec)
		durframe = max(int(durationsec/self.hopLengthSec), 1) # at least one frame!
		for d in self.rawData[sffile]['load']:
			dataDest[d] = self.rawData[sffile]['load'][d][startframe:startframe+durframe]
		if envelopeMask != None:
			for d in dataDest: 
				if d in self.amplitudedescriptors:
					dataDest[d] *= envelopeMask
	########################################################
	########################################################
	def doneSdifResource(self, handle, sffile):
		return
		sffile = os.path.abspath(sffile)
		self.rawData[sffile]['handle'].remove(handle)
		if len(self.rawData[sffile]['handle']) == 0:
			del self.rawData[sffile]
			#print "REMOVING SDIF RESOURCE", sffile



########################################################
########################################################







def listToCheckSum(items):
	import hashlib
	m = hashlib.md5()
	for item in items:
		m.update(str(item))
	output = m.hexdigest()[0:12] # ahh, fuck it
	return output


######################################################################
# audioguide-descriptor-name: (filetype, frame, matrix, row, column)
######################################################################
agDescriptToSdif = {
'clusterid': ('cluster', '1CST', '1CST', 0, 0),
# mfccs get automatically added here based on numbBands
# cluster_cents get automatically added here based on numbClust
# peakfrqs get automatically added here based on numbPeaks
# peakamps get automatically added here based on numbPeaks
# partialfrqs get automatically added here based on maxpartials
# partialamps get automatically added here based on maxpartials
'bandfluctuationstrength': ('descriptors', '1DSC', '1BFL', 0, 0),
'bandroughness': ('descriptors', '1DSC', '1BRG', 0, 0),
'centroid': ('descriptors', '1DSC', '1SCN', 0, 0),
'chroma0': ('descriptors', '1DSC', '1CHR', 0, 0),
'chroma1': ('descriptors', '1DSC', '1CHR', 1, 0),
'chroma2': ('descriptors', '1DSC', '1CHR', 2, 0),
'chroma3': ('descriptors', '1DSC', '1CHR', 3, 0),
'chroma4': ('descriptors', '1DSC', '1CHR', 4, 0),
'chroma5': ('descriptors', '1DSC', '1CHR', 5, 0),
'chroma6': ('descriptors', '1DSC', '1CHR', 6, 0),
'chroma7': ('descriptors', '1DSC', '1CHR', 7, 0),
'chroma8': ('descriptors', '1DSC', '1CHR', 8, 0),
'chroma9': ('descriptors', '1DSC', '1CHR', 9, 0),
'chroma10': ('descriptors', '1DSC', '1CHR', 10, 0),
'chroma11': ('descriptors', '1DSC', '1CHR', 11, 0),
'crest0': ('descriptors', '1DSC', '1SCM', 0, 0),
'crest1': ('descriptors', '1DSC', '1SCM', 1, 0),
'crest2': ('descriptors', '1DSC', '1SCM', 2, 0),
'crest3': ('descriptors', '1DSC', '1SCM', 3, 0),
'decrease': ('descriptors', '1DSC', '1SDE', 0, 0),
'energyenvelope': ('descriptors', '1DSC', '1EEV', 0, 0),
'f0': ('descriptors', '1DSC', '1FQ0', 0, 0),
'flatness0': ('descriptors', '1DSC', '1SFM', 0, 0),
'flatness1': ('descriptors', '1DSC', '1SFM', 1, 0),
'flatness2': ('descriptors', '1DSC', '1SFM', 2, 0),
'flatness3': ('descriptors', '1DSC', '1SFM', 3, 0),
'fluctuationstrength': ('descriptors', '1DSC', '1FLS', 0, 0),
'harmoniccentroid': ('descriptors', '1DSC', '1HCN', 0, 0),
'harmonicdecrease': ('descriptors', '1DSC', '1HDE', 0, 0),
'harmonicdeviation': ('descriptors', '1DSC', '1HSD', 0, 0),
'harmonicenergy': ('descriptors', '1DSC', '1HEN', 0, 0),
'harmonickurtosis': ('descriptors', '1DSC', '1HKU', 0, 0),
'harmonicoddevenratio': ('descriptors', '1DSC', '1HOE', 0, 0),
'harmonicrolloff': ('descriptors', '1DSC', '1HRO', 0, 0),
'harmonicskewness': ('descriptors', '1DSC', '1HSK', 0, 0),
'harmonicslope': ('descriptors', '1DSC', '1HSL', 0, 0),
'harmonicspread': ('descriptors', '1DSC', '1HSP', 0, 0),
'harmonictristimulus0': ('descriptors', '1DSC', '1HTR', 0, 0),
'harmonictristimulus1': ('descriptors', '1DSC', '1HTR', 1, 0),
'harmonictristimulus2': ('descriptors', '1DSC', '1HTR', 2, 0),
'harmonicvariation': ('descriptors', '1DSC', '1HVA', 0, 0),
'inharmonicity': ('descriptors', '1DSC', '1INH', 0, 0),
'kurtosis': ('descriptors', '1DSC', '1SKU', 0, 0),
'loudness': ('descriptors', '1DSC', '1LDN', 0, 0),
'noisecentroid': ('descriptors', '1DSC', '1NCN', 0, 0),
'noisedecrease': ('descriptors', '1DSC', '1NDE', 0, 0),
'noiseenergy': ('descriptors', '1DSC', '1NEN', 0, 0),
'noisekurtosis': ('descriptors', '1DSC', '1NKU', 0, 0),
'noiserolloff': ('descriptors', '1DSC', '1NRO', 0, 0),
'noiseskewness': ('descriptors', '1DSC', '1NSK', 0, 0),
'noiseslope': ('descriptors', '1DSC', '1NSL', 0, 0),
'noisespread': ('descriptors', '1DSC', '1NSP', 0, 0),
'noisevariation': ('descriptors', '1DSC', '1NVA', 0, 0),
'noisiness': ('descriptors', '1DSC', '1NSN', 0, 0),
'perceptualcentroid': ('descriptors', '1DSC', '1PCN', 0, 0),
'perceptualdecrease': ('descriptors', '1DSC', '1PDE', 0, 0),
'perceptualdeviation': ('descriptors', '1DSC', '1PSD', 0, 0),
'perceptualkurtosis': ('descriptors', '1DSC', '1PKU', 0, 0),
'perceptualoddtoevenratio': ('descriptors', '1DSC', '1POE', 0, 0),
'perceptualrolloff': ('descriptors', '1DSC', '1PRO', 0, 0),
'perceptualskewness': ('descriptors', '1DSC', '1PSK', 0, 0),
'perceptualslope': ('descriptors', '1DSC', '1PSL', 0, 0),
'perceptualspread': ('descriptors', '1DSC', '1PSP', 0, 0),
'perceptualtristimulus0': ('descriptors', '1DSC', '1PTR', 0, 0),
'perceptualtristimulus1': ('descriptors', '1DSC', '1PTR', 1, 0),
'perceptualtristimulus2': ('descriptors', '1DSC', '1PTR', 2, 0),
'perceptualvariation': ('descriptors', '1DSC', '1PVA', 0, 0),
'power': ('descriptors', '1DSC', '1NRG', 0, 0),
'relativespecificloudness': ('descriptors', '1DSC', '1RSL', 0, 0),
'rolloff': ('descriptors', '1DSC', '1SRO', 0, 0),
'roughness': ('descriptors', '1DSC', '1RGH', 0, 0),
'sharpness': ('descriptors', '1DSC', '1SHA', 0, 0),
'skewness': ('descriptors', '1DSC', '1SSK', 0, 0),
'slope': ('descriptors', '1DSC', '1SSL', 0, 0),
'spread': ('descriptors', '1DSC', '1SSP', 0, 0),
'variation': ('descriptors', '1DSC', '1SVA', 0, 0),
'zeroCross': ('descriptors', '1DSC', '1ZCR', 0, 0),
'logAttackTime-seg': ('descriptors', '1LAT', 'IDSC', 0, 0),
'temporalIncrease-seg': ('descriptors', '1TIN', '1EEV', 0, 0),
'temporalDecrease-seg': ('descriptors', '1TDE', '1EEV', 0, 0),
'temporalCentroid-seg': ('descriptors', '1TCN', '1EEV', 0, 0),
#'effDur-seg': ('1EFD', '1EEV', 0, 0),
}



def sdif_testprint(sdfifile):
	sdfh = pysdif.SdifFile(sdfifile)
	print sdfifile
	for f in sdfh:
		print f.time, f.signature
		for m in f:
			print '\t', m.signature, '->', m.get_data(copy=False)
	# close this sdif file handle
	sdfh.close()
