try:
	from aubio.aubioclass import * # for new_fvec
except ImportError, err:
	pass
from math import log10
from numpy import array	
import sys 
import os
import random
import subprocess

def exit(*args):
	ALERT_ON_ERROR = True
	if ALERT_ON_ERROR:  # beep?
		sys.stdout.write('\a')
		sys.stdout.flush()
	print "\n"
	sys.exit()


def error(errorType, errorData):
	import sys
	if errorType == 'path':
		print "\n\n\n"+bold("FILE ERROR")+": required file not found in system ->", errorData
		exit(5)
	elif errorType == 'ops':
		print "\n\n\n"+bold("OPTIONS ERROR")+":", errorData
		exit(5)
	elif errorType == 'commandline':
		print "\n\n\n"+bold("COMMAND LINE CALL ERROR")+":", errorData
		exit(5)
	else:
		print "\n\n\n"+bold(errorType.upper()+" ERROR")+': '+errorData
		exit(5)



def bold(string):
	return "\033[1m"+str(string)+"\033[0;0m"


def getTimeStamp(file):
	return os.stat(file).st_ctime

def checkIfFileIsNewer(file1, file2):
	if not os.path.exists(file2):
		return True
	if getTimeStamp(file1) > getTimeStamp(file2): return True
	else: return False
	

def quantize(value, interval):
	interval = float(interval)
	if interval == 0:
		return value
	else:
		if value < 0: interval = -interval
		return int((value+(interval/2.0))*(1/interval))*interval


def quantizeNoteSequence(inputNotes, quant):
	if quant not in [None, 0]: # then quantize the score
		startTimes = []
		for entry in inputNotes:
			if str(entry[0])[0] == ";": continue # just a comment
			startTimes.append(float(entry[0]))
		quantizes = sliceListIntoEvenlySpacedLists(startTimes, quant, 0, max(startTimes)+1)
		# do histogram to find most common attack point for each window...
		for idx, noteList in enumerate(quantizes): 
			if len(quantizes[idx]) > 0: quantizes[idx] = histogram(noteList)[0][1] # only if there are notes
		# make the changes
		for entry in inputNotes:
			if str(entry[0])[0] == ";": continue # just a comment
			idx = int(float(entry[0])/quant)
			entry[0] = quantizes[idx] # change start time of this note
	return inputNotes


def permutations(iterable, r=None):
    # permutations('ABCD', 2) --> AB AC AD BA BC BD CA CB CD DA DB DC
    # permutations(range(3)) --> 012 021 102 120 201 210
    pool = tuple(iterable)
    n = len(pool)
    r = n if r is None else r
    if r > n:
        return
    indices = range(n)
    cycles = range(n, n-r, -1)
    yield tuple(pool[i] for i in indices[:r])
    while n:
        for i in reversed(range(r)):
            cycles[i] -= 1
            if cycles[i] == 0:
                indices[i:] = indices[i+1:] + indices[i:i+1]
                cycles[i] = n - i
            else:
                j = cycles[i]
                indices[i], indices[-j] = indices[-j], indices[i]
                yield tuple(pool[i] for i in indices[:r])
                break
        else:
            return


def ampToDb(amp):
	return (20*log10(max(amp, 0.0000000000001))) # amp to dB


def dbToAmp(db):
	return pow(10, db/20.0) # dB to amp


def maxIndex(listy):
	m = max(listy)
	for i, v in enumerate(listy):
		if m == v:
			maxy = i
			break
	return maxy


def minIndex(listy):
	m = min(listy)
	for i, v in enumerate(listy):
		if m == v:
			minny = i
			break
	return minny


def frq2Midi(frq):
	from math import log
	if frq == 0.0: frq = 0.001
	return round(12*log(frq/440.0), 2)+69

	
def list2Fvec(listy):
	fvec = new_fvec(len(listy), 1)
	for i in range(len(listy)):
		fvec_write_sample(fvec, listy[i], 0, i)
	return fvec


def fvec2List(fvec, length):
	listy = []
	for i in range(length):
		listy.append(fvec_read_sample(fvec, 0, i))
	return listy


def limit(inValue, low, high):
	if high != None:
		if inValue > high: inValue = high
	if low != None:
		if inValue < low: inValue = low
	return inValue



def interpolateVal(scalar, low, high):
	return ((high-low)*scalar)+low


def calcRollOffDataPoint(metric, dataSlot, idx, dataPoint, value, modType): # used by tgt.calcFeatureWarp
	ceiling = 3 # when to stop modifying target data
	slope = 0.5 # the slope of the rolloff - 1=linear, 2=exponenetial, etc.
	dataPointMod = eval(str(dataPoint)+modType+str(value))
	tgt.data[dataSlot][metric][idx] = softClip(dataPointMod, 1.5, ceiling, slope)


def softClip(data, startClip, ceiling, slope):
	# data, starting point to appply fn, ceiling which gives no change, slope of clipping function
	if abs(data) < startClip: return data # pass in under startClip
	elif abs(data) >= ceiling: return ceiling # yields ceiling if data is higher than it
	# else apply interpolate clipping ...
	clipLoc = (abs(data)-startClip)/(ceiling-startClip)**slope
	dataClipped = interpolateVal(clipLoc, startClip, ceiling)
	if data < 0: dataClipped *= -1 # make negative if input was below 0
	return data


def product(*args, **kwds):
    # product('ABCD', 'xy') --> Ax Ay Bx By Cx Cy Dx Dy
    # product(range(2), repeat=3) --> 000 001 010 011 100 101 110 111
    pools = map(tuple, args) * kwds.get('repeat', 1)
    result = [[]]
    for pool in pools:
        result = [x+[y] for x in result for y in pool]
    for prod in result:
        yield tuple(prod)


def combinations(iterable, r):
    # combinations('ABCD', 2) --> AB AC AD BC BD CD
    # combinations(range(4), 3) --> 012 013 023 123
    pool = tuple(iterable)
    n = len(pool)
    if r > n:
        return
    indices = range(r)
    yield tuple(pool[i] for i in indices)
    while True:
        for i in reversed(range(r)):
            if indices[i] != i + n - r:
                break
        else:
            return
        indices[i] += 1
        for j in range(i+1, r):
            indices[j] = indices[j-1] + 1
        yield tuple(pool[i] for i in indices)




def getDynamicFromFilename(file, notFound=-1000):
	TYPICAL_DYNAMICS = {'pp': -50, 'p': -40, 'mp': -34, 'mf': -24, 'f': -20, 'ff': -10}
	MAP_DYNAMICS = {'ppmfpp': 'mf', 'pfp': 'f', 'f-ff': 'f','fp': 'f', 'ppff':'ff', 'ffpp': 'ff', 'slap': 'pp', 'sfz': 'f', 'p1': 'p', 'p2': 'pp', }
	ALL_DYN_KEYS = TYPICAL_DYNAMICS.keys()
	ALL_DYN_KEYS.extend(MAP_DYNAMICS.keys())
	SPLIT_STRINGS = ['-', '_', '.', '|'] # in order of likelihood
	NOTHING_YET = True
	whichStr = 0
	dynamic = "fuck"
	while NOTHING_YET:
		if whichStr > len(SPLIT_STRINGS)-1:
			break
		splitStr = SPLIT_STRINGS[whichStr]
		fileHead = os.path.split(file)[1]
		strings = os.path.split(os.path.splitext(file)[0])[1].split(splitStr)
		strings.reverse() # they tend to be at the end of the file
		for s in strings:
			if s in ALL_DYN_KEYS:
				dynamic = s
				NOTHING_YET = False
				break
		whichStr += 1
	if dynamic == 'fuck':
		return notFound
	elif MAP_DYNAMICS.has_key(dynamic):
		return TYPICAL_DYNAMICS[MAP_DYNAMICS[dynamic]]
	else:
		return TYPICAL_DYNAMICS[dynamic]


def reqFile(path):
	if not os.path.isfile(path): error('fileNotFound', path)
	else: return path # return it if it exists


def reqDir(path):
	if not os.path.isdir(path): os.makedirs(path)


def matchString(testString, matchStr, caseSensative=True):
	if not caseSensative:
		testString = testString.lower()
		matchStr = matchStr.lower()
	if testString.find(matchStr) == -1: return False
	else: return True


def readAudacityLabelFile(path, wholeFile, printVerbose):
	if wholeFile: return [[0, None]] # just use the whole soundfile, no segmentation
	timeList = []
	audacityLabels = open(path, 'r')
	lines = audacityLabels.readlines()
	# in case it was made in audacity and has a '\r' for a carriage return!
	try:
		if len(lines) == 1 and lines[0].find('\r') != -1:
			lines = lines[0].split('\r') 
		for idx, line in enumerate(lines):
			if len(line) == 0: continue
			line = line.split('\t')
			start = float(line[0])
			end = float(line[1])
			if start == end: # then we'll assume these are start times and infer end time...
				if idx != len(lines): end = None
				else: end = float(lines[idx+1].split()[1])-0.05
			writeLine = [start, end]
			writeLine.extend(line[2:])
			timeList.append( writeLine )
	except ValueError:
		print ValueError, "on line", idx, "in", path

	return timeList


def plot(name, data, labeled, segs=False, normalize=False, height=15, width=25, extraText=[]):
	import matplotlib.pyplot as plt
	fig = plt.figure(figsize=(width, height))
	for item in data:
		if normalize:
			item = array(item)
			miny, maxy = min(item), max(item)
			item = (((item-miny)/(maxy-miny))*0.9)+0.1
			item = list(item)
		plt.plot(range(len(item)), item, ls='steps-post', lw=1, label=labeled)
	if labeled != None:
		leg = plt.legend(labeled,'upper right', shadow=False)
		frame  = leg.get_frame()  
		frame.set_facecolor('0.90')    # set the frame face color to light gray
	if segs != False:
		for seg in segs: # plot segmentation
			plt.axvspan(seg[0], seg[1], facecolor='r', alpha=1, ec='r', lw=1)
	txtLoc = [0.01, 0.98]
	for txt in extraText:
		plt.figtext(txtLoc[0], txtLoc[1], txt)
		txtLoc[1] -= 0.06
	plt.savefig(name)
	plt.close()


def makeDescriptorGraphable(descript, tgt):
	import numpy as np
	if descript.endswith('-seg'):
		plotData = np.zeros(tgt.lengthInFrames-1)
		miny = +1000000
		for point in tgt.data['raw'][descript][1:]:
			if point == None: continue
			if point < miny: miny = point
		plotData.fill(miny) # fill in minimum value for the whole time
		for st,ed in tgt.segFrames:
			for i in range(st,ed):
				plotData[i] = tgt.data['raw'][descript][st]
	else:
		plotData = np.zeros(tgt.lengthInFrames-1)
		plotData.fill(np.min(tgt.data['raw'][descript][1:])) # fill in minimum value for the whole time
		plotData[0] = np.min(tgt.data['raw'][descript][1:])-100
		plotData[-1] = np.max(tgt.data['raw'][descript][1:])+100
		for st,ed in tgt.segFrames:
			for i in range(st,ed):
				plotData[i] = tgt.data['raw'][descript][i]
	return plotData



def plotMetric(handleList):
	plotDir = ops.SCRIPT_PATH+'/plots'
	for idx, handle in enumerate(handleList):
		labels = []
		data = []
#handle.data['raw'].keys()
		for k in ['centroid', 'power']:
			if k not in ['concatAmps', 'negMelamps', 'samples', 'mfcc', 'rms', 'FFT', 'melamps', 'specSlope', 'kurtosis', 'skewness', 'flatness'] and k[-3:] != 'seg':
				#if type(handle.data['raw'][k][0]) in [float, 'numpy.float64']:
				d = array(handle.data['raw'][k])
				d = (d-min(d))/(max(d)-min(d))
				d = list(d) # quick and dirty tmp normalize
				data.append(d)
				labels.append(k)
		if os.path.exists(plotDir) == False: os.makedirs(plotDir)
		thisPlot = plotDir+'/'+str(handle.name)+'-'+str(idx)+'.png'
		print thisPlot
		plot(thisPlot, data, labels)


#def waterfallPlot(name, TwoDArrays):
#	import numpy
#	from enthought.mayavi.mlab import figure, surf, savefig
#	fig = figure(fgcolor=(0,0,0), bgcolor=(1,1,1))
#	for TwoDArray in TwoDArrays:
#		surf(TwoDArray, colormap='Greys') #
#	savefig(name)


def plotMatches(handle1, handle2, handle1start, handle2start, handle1peak, handle2peak, duration, counter, dataType, normalize=False, metrics=None):
	import os
	normalize = True
	plotFileName = os.path.join(ops.SCRIPT_PATH,"output/segs",str(counter)+'.png')
	dataArrays = []
	dataNames = []
	extraTexts = ['duration = '+trunc(duration*ag.frameSec, 3)]
	for d in metrics: # normalize
		if d.seg:
			extraTexts.append( d+": "+str(trunc(float(handle1.data[dataType][d.name][handle1start]), 1))+" vs "+str(trunc(float(handle2.data[dataType][d.name][handle2start]), 1)) )
			continue
		for handle, time in [(handle1, handle1start), (handle2, handle2start)]:
			listy = array(handle.data[dataType][d.name][time:time+duration])
			if normalize:
				miny, maxy = min(listy), max(listy)
				listy = (((listy-miny)/(maxy-miny))*0.9)+0.1
			dataArrays.append(listy)
			dataNames.append(d+'-'+os.path.splitext(handle.name)[0])
	plot(plotFileName, dataArrays, dataNames, height=3.5, width=15, extraText=extraTexts)


def printDict(heading, dict, postLevel, p):
	p.middle(heading, postLevel)
	for key, val in dict.iteritems():
		p.post(str(key)+" -> ${RED}"+str(val)+"${NORMAL}", postLevel)
	p.post("", postLevel)



def executeCommand(args):
	import subprocess, os
	sub = subprocess.Popen(args, shell=True)
	exitStatus = os.waitpid(sub.pid, 0)[1]
	if exitStatus != 0:
		p.post(' '.join(args) + " died with " + str(exitStatus), 1)
		exit(10)


def popen_execute_command(commandArgs, exitOnError=True, stdoutReturnDict=None):
	try:
		p = subprocess.Popen(commandArgs, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
	except OSError:
		error('commandline', 'Command line call failed: \n\n"%s"'%' '.join(commandArgs))
	out, err = p.communicate()
	# test for bad exit status
	if exitOnError and err not in [0, '']:
		error('commandline', 'AudioGuide command line call failed: \n"%s"%s%s'%(' '.join(commandArgs), '\n--------\n\n', ''.join(out)))	
	if stdoutReturnDict == None: return None
	dictStore = {}
	# fill output dict if requested
	for o in out.split('\n'):
		o = o.split()
		if len(o) > 1:
			for (str, loc), (key, valloc, valtype) in stdoutReturnDict.iteritems():
				if o[loc] == str:
					dictStore[key] = o[valloc]
					if valtype == int: dictStore[key] = int(dictStore[key])
					if valtype == float: dictStore[key] = float(dictStore[key])
	return dictStore




def executeCommandQuite(args):
	args.extend(['>', '/tmp/out', '2>&1'])
	executeCommand(args)



def isNumber(s):
	try:
		float(s)
		return True
	except ValueError: return False



def trunc(f, n):
    '''Truncates/pads a float f to n decimal places without rounding'''
    slen = len('%.*f' % (n, f))
    return str(f)[:slen]


def settingPrint(print1, string):
	string = str(string)
	spaces = (ops.PRINT_LENGTH-2-len(string)-len(print1))*'.'
	print print1+" "+spaces+" "+string		



def chain(iterables):
	output = []
	for it in iterables:
		output.extend(it)
	return output



def histogram(l):
	# returns sorted list histogram with highest value first.
	d = {}
	for i in l:
		if d.has_key(i): d[i] += 1
		else: d[i] = 1
	sorty = []
	for k, v in d.iteritems():
		sorty.append((v, k))
	sorty.sort()
	sorty.reverse()
	return sorty



def sliceListIntoEvenlySpacedLists(l, sliceSize, lowVal, highVal):
	# for use with quantize and histograms...
	l.sort()
	numberSlices = int((highVal-lowVal+1)/sliceSize) # add an extra second just to be sure
	slices = []
	for i in range(numberSlices): # fill with enough empty lists
		slices.append([])
	for item in l:
		slices[int((item-lowVal)/sliceSize)].append(item)
	return slices




def testCorpusLimits(sfPrelimHandleList, descriptorList, p, ag):
	for d in descriptorList:
		if d.name.find('-seg') == -1:
			error('options', 'limit descriptors only works with segmented methods!')
			exit()
	###@@###
	#import metrics
	limitData = {}
	maxMin = {'global': {}}
	orderings = {'global': {}}
	for sf in sfPrelimHandleList:
		p.post(sf.name+"@"+str(trunc(sf.startSec, 2))+" - computing limit anals", inc=True)
		if not maxMin.has_key(sf.name):
			maxMin[sf.name] = {}
			orderings[sf.name] = {}
		limitData[sf] = {}
		for d in descriptorList: limitData[sf][d.name] = sf.data['raw'][d.name][0]
	# find max and min of each limit descriptor
	for key in maxMin:
		for idx, d in enumerate(descriptorList):
			if key == 'global':
				accumGlobal = []
				orderGlobal = []
				for handle, dataDict in limitData.iteritems():
					accumGlobal.append(dataDict[d.name])
					orderGlobal.append((dataDict[d.name], handle))
				maxMin[key][d.name] = ((min(accumGlobal), max(accumGlobal)))
				orderGlobal.sort()
				orderings[key][d.name] = []
				for (val, handle) in orderGlobal: orderings[key][d.name].append(handle)
			else: # its local
				accumLocal = []
				orderLocal = []
				for handle, dataDict in limitData.iteritems():
					if handle.name != key: continue
					accumLocal.append(dataDict[d.name])
					orderLocal.append((dataDict[d.name], handle))
				maxMin[key][d.name] = ((min(accumLocal), max(accumLocal)))
				orderLocal.sort()
				orderings[key][d.name] = []
				for (val, handle) in orderLocal: orderings[key][d.name].append(handle)
	for sf in sfPrelimHandleList:
		limitData[sf] = sf.testLimits(limitData, maxMin, orderings) # get limit metric values






def getDirListOnlyExt(path, recur, extensions):
	import os
	# if recursive..
	if recur:
		output = []
		for dirname, dirnames, filenames in os.walk(path):
			for filename in filenames:
				output.append( os.path.abspath(os.path.join(dirname, filename)) )
	else:
		# if just whats in that folder..
		output = os.listdir(path)
	# filter files by extension - case insensative!
	validFiles = []
	for outty in output:
		isValid = False
		fileExt = os.path.splitext(outty)[1]
		for ext in extensions:
			if ext.lower() == fileExt.lower():
				isValid = True
				break
		if isValid: validFiles.append(outty)
	return validFiles





def getTransposition(tgt, chosen, thisTargetSegmentLength, quantizef):
	# NADA
	if chosen.transMethod[0] == 'none':
		output = 0
	# RANDOM RANGE
	elif chosen.transMethod[0] == 'randomRange':
		output = random.uniform(chosen.transMethod[1][0], chosen.transMethod[1][1])
	# SMART
	elif chosen.transMethod[0] == 'smart':
		SMART_TRANSPOSE_PC = True # transpose by pitch class, not just pitch
		PITCH_TOLERANCE = 6 # in +/- semitones
		tgtPitch = tgt.onFlySegmentCalc('f0-seg', tgt.timeInFrames, thisTargetSegmentLength, 'raw') 
	#	print tgtPitch, tgt.calcFeatureWarp('f0-seg', tgtPitch, 'raw', tgt.timeInFrames, thisTargetSegmentLength)
	#	srcPitch = chosen.data['raw']['f0-seg'][0]
		srcPitch = chosen.data['raw']['MIDIPitch-seg'][0]
		if SMART_TRANSPOSE_PC:
			output = 1
			tgtPc = tgtPitch%12
			srcPc = srcPitch%12
			output = tgtPc-srcPc
			if output > 6: output = (output-12)*-1
			if output < -6: output = (output+12)*-1
		else:
			if abs(tgtPitch-srcPitch) <= PITCH_TOLERANCE:
				output = tgtPitch-srcPitch
	# MARKOV
	elif chosen.transMethod[0] == 'markov':
		output = makeMarkovPitchChoice(chosen.transMethod[1])
	elif chosen.transMethod[0] == 'matchTgtTemporality':
	# match the length of tgt segment
		scaleTgtDur = 1.3 # to provide a little overlap
		minTrans = 0.5
		maxTrans = 2.
		rawTrans = chosen.data['raw']['effDur-seg'][0]/float(thisTargetSegmentLength*1.1)
		output = np.clip(rawTrans, minTrans, maxTrans)
	return quantize(output, quantizef)


def makeMarkovPitchChoice(incomingPitch):
	global markov1, markov1IntMap
	if tgt.lastMarkovIntMapChoice == None: # INIT ME
		#     	m2nd	M2nd	m3rd	M3rd	TT
		m2nd =[ 30,		10,		10,		20,		10]
		M2nd =[ 10,		0,		2,		3,		3]
		m3rd =[ 0,		5,		5,		0,		20]
		M3rd =[ 30,		2,		0,		1,		3]
		TT   =[ 60,		10,		0,		10,		10]
		markov1 = [m2nd, M2nd, m3rd, M3rd, TT]
		markov1IntMap = [1, 2, 3, 4, 6]
		for likelihood in markov1:
			total = 0.0
			for i in range(0, len(likelihood)):
				total = total+likelihood[i]
			for i in range(0, len(likelihood)): # make sure all lists total 1...
				likelihood[i] = likelihood[i]*(1/total)
		# write first int val
		tgt.lastMarkovIntMapChoice = int(random.uniform(0, len(likelihood))) # set it if this is the first note...
		tgt.lastMarkovPitchChoice = incomingPitch
		return 0
	else: # make a markov call...
		number = random.uniform(0, 1)
		accum = 0
		i = 0
		while accum < number:
		    accum += markov1[int(tgt.lastMarkovIntMapChoice)][int(i)]
		    i += 1
		thisIntMapChoice = i-1
		thisInterval = markov1IntMap[thisIntMapChoice]
		thisTargetPitch = thisInterval+incomingPitch
#		print ""
#		print ""
#		print ""
#		print "INCOMING PITCH:", incomingPitch, "INCOMING PC:", incomingPitch%12
#		print "CHOSEN INTERVAL:", thisInterval, "LAST PITCH:", tgt.lastMarkovPitchChoice
#		print "TARGET PITCH:", (tgt.lastMarkovPitchChoice+thisInterval), "TARGET PC:", (tgt.lastMarkovPitchChoice+thisInterval)%12
		subtractPC = (tgt.lastMarkovPitchChoice+thisInterval)%12-(incomingPitch%12)
		addPC = (tgt.lastMarkovPitchChoice+thisInterval)%12+(incomingPitch%12)
		if abs(addPC) <= abs(subtractPC): trans = addPC
		else: trans = subtractPC
		while trans > 6: trans -= 12
		while trans < -6: trans += 12
#		print "CHOSEN TRANS:", trans
#		print ""
		tgt.lastMarkovPitchChoice = incomingPitch+trans
		return trans




def subtractEval(tgtHandle, sampleHandle, posEnergy, negEnergy):
	if ops.ANALIZE_MEL_AMP_DIFF:
		energyRatio = []
		for i in range(len(posEnergy)):
			if posEnergy[i] == 0:
				energyRatio.append(10000)
			else:
				energyRatio.append(abs(negEnergy[i])/posEnergy[i])
		return simcalc.energyWeightSingle(energyRatio, sampleHandle.data['raw']['power'])
	else:
		return None



def melBandAvgDiff(bands1, bands2):
	numbFrames = min([len(bands1),len(bands2)])
	avgDiff = 0
	for i in range(numbFrames): # should skip frames if INCREMENT is not 1?
		residualFrame = array(bands1[i])-array(bands2[i])
		avgDiff = avgDiff+sum(abs(residualFrame))
	return avgDiff/float(numbFrames)
#print melBandAvgDiff([[1, 1], [1, 1], [1, 1], [1, 1]], [[0.5, 0.5], [0.5, 0.5], [2, 2], [2, 2]])


#def path.locate(*components):
#	if os.path.isabs(components[-1]): return components[-1] # if user supplied var is a valid path, use it
#	else: return os.path.join(ops.SCRIPT_PATH, *components) # otherwise make into a relative path...




def which(program):
    import os
    def is_exe(fpath):
        return os.path.exists(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file
    return None



def printableList(listOfLists):
	# find max list length
	maxLength = 0
	for line in listOfLists:
		if len(line) > maxLength: maxLength = len(line)
	# make a list of the longest string lengths for each field
	maxStrLen = [0]*maxLength
	for line in listOfLists:
		if line[0][0] == ";": continue # just a comment
		for (idx, val) in enumerate(line):
			if len(val) > maxStrLen[idx]: maxStrLen[idx] = len(val)
	outputListOfLists = []
	# makes sure that each param has a fixed number of chars
	for line in listOfLists:
		if line[0][0] == ";":
			outputListOfLists.append(' '.join(line)) # comment
			continue
		outStr = ""
		for idx, val in enumerate(line):
			outStr += str(val)+(" "*(maxStrLen[idx]-len(val)+1))
		outputListOfLists.append(outStr+'\n')
	return outputListOfLists


	
def getTargetCorpusChecksum(ops, tgt, agSourceHandles, descriptorNormList):
	clusterChecksumItems = [tgt.name, tgt.startSec, tgt.endSec, ops.NORMALIZE_NUMBER_CLUSTERS, ops.TARGET_ONSET_THRESHOLD, ops.TARGET_ONSET_DESCRIPTORS, ops.TARGET_OFFSET_RISE_RATIO, ops.TARGET_SEGMENT_DROP_RATIO, ]
	for handle in agSourceHandles:
		clusterChecksumItems.extend([handle.name, handle.startSec, handle.endSec])
	for d in descriptorNormList: clusterChecksumItems.append(d.name)
	return listToCheckSum(clusterChecksumItems)


def getTargetCorpusTimbreRemapChecksum(tgt, agSourceHandles, descriptorList):
	clusterChecksumItems = [tgt.name, tgt.scaleDb]
	for tup in enumerate(tgt.segFrames): clusterChecksumItems.extend(list(tup))
	for handle in agSourceHandles:
		clusterChecksumItems.extend([handle.name, handle.startSec, handle.endSec, handle.scaleDb])
	for d in descriptorList:
		clusterChecksumItems.extend([d.name, d.weight])
	return listToCheckSum(clusterChecksumItems)





def listToCheckSum(items):
	import hashlib
	m = hashlib.md5()
	for item in items:
		m.update(str(item))
	output = m.hexdigest()
	return output
	