#!/usr/bin/env python2.7
import sys, os, random

from optparse import OptionParser
parser = OptionParser(usage="usage: %prog [options] soundfile")
parser.set_defaults(THRESHOLD=-40)
parser.set_defaults(RISE=1.3)
parser.set_defaults(DB=12)
parser.set_defaults(MINIMUM_SEG=0.05)
parser.set_defaults(MAXIMUM_SEG=4)
parser.set_defaults(GRAIN_SIZE=None)
parser.set_defaults(grainRange=None)

parser.set_defaults(GRAIN_OVERLAP=1)
parser.set_defaults(OUTPUT_FILE='')

parser.add_option("-t", "--threshold", action="store", dest="THRESHOLD", type="float", help="set the threshold for segmentation.  a value from -100 to 0 where a lower value make onsets happen more frequently.  default=-40")
parser.add_option("-r", "--rise", action="store", dest="RISE", type="float", help="set the rise factor for segment onsets.  a higher value makes onsets happen more easily.  default=1.1")
parser.add_option("-d", "--db", action="store", dest="DB", type="float", help="set the db above minimum segment offsets.  a low value (6) means that an offset won't happen until 6dB above the lowest amplitude in the soundfile.  default=12")
parser.add_option("-g", "--grain", action="store", dest="GRAIN_SIZE", type="float", help="slice this soundfile into evenly spaced windows of N seconds.  overrides a normal threshold method.  by default is disabled.")
parser.add_option("-b", "--grainRange", action="store", dest="grainRange", type="str", help="")

parser.add_option("-o", "--grainOverlap", action="store", dest="GRAIN_OVERLAP", type="float", help="set the grain overlap.  by default is disabled.")
parser.add_option("-e", "--exhaustive", action="store_true", dest="exhaustive", default=False)
parser.add_option("-s", "--minimum", action="store", dest="MINIMUM_SEG", type="float", help="set the minimum segment length in seconds.  segments will be forced to be this duration or greater.  default=0.05")
parser.add_option("-l", "--maximum", action="store", dest="MAXIMUM_SEG", type="float", help="set the maximum segment length in seconds.  segments will be forced to be this duration or shorter.  default=4")
parser.add_option("-f", "--file", action="store", dest="OUTPUT_FILE", type="str", help="set the name of the output file.  by default it is the soundfile name + '.txt'")


(options, args) = parser.parse_args()


import audioguide
agPath = os.path.split(audioguide.__file__)[0]
scriptPath = os.path.dirname(os.path.realpath(__file__))

ag = audioguide.main(os.path.dirname(os.path.realpath(__file__)), "defaults.py", loadOptionsFromFile=os.path.join(agPath, 'defaults.py'), printName="Segment Soundfile")
for file in args:
	# test if its an audio file
	fileExtension = os.path.splitext(file)[1]
	isValidSoundfile = False
	for ext in audioguide.ops.SOUNDFILE_EXTENSIONS:
		if ext.lower() == fileExtension.lower():
			isValidSoundfile = True
			break
	if not isValidSoundfile: print("\n\n%s is not a valid soundfile!!!!\n\n"%(file))
	# segment file, write audacity-style label txt file and exit
	audioguide.ops.CORPUS = []
	audioguide.ops.TARGET_USE_DATABASE = False
	audioguide.ops.TARGET_OFFSET_DB_ABOVE_MIN = options.DB
	audioguide.ops.TARGET_OFFSET_RISE_RATIO = options.RISE
	if options.GRAIN_SIZE == None: 
		audioguide.ops.TARGET_MIN_SEGMENT_LENGTH = options.MINIMUM_SEG
		audioguide.ops.TARGET_MAX_SEGMENT_LENGTH = options.MAXIMUM_SEG
	else: # granular
		audioguide.ops.TARGET_MIN_SEGMENT_LENGTH = 0.01
		audioguide.ops.TARGET_MAX_SEGMENT_LENGTH = 10000
	audioguide.ops.TARGET_ONSET_THRESHOLD = options.THRESHOLD
	audioguide.ops.TARGET = [file, None, "end"]
	#audioguide.ops.SAMPLE_RATE = audioguide.readsf.fileSR(audioguide.ops.TARGET[0]) # change sample rate to match tgt
	audioguide.ops.DESCRIPTORS = {'power': 1}
	audioguide.ops.TARGET_SEGMENT_LENGTH_DROP_METHOD = 'absDb' # absDb or relative
	audioguide.ops.TARGET_ONSET_ENVELOPE = 'orig' # orig=unsubtracted, raw=subtracted
	audioguide.ops.TARGET_OFFSET_ENVELOPE = 'orig' # orig=unsubtracted, raw=subtracted
	audioguide.ops.TARGET_FADE_IN_SEC = 0
	audioguide.ops.TARGET_FADE_OUT_SEC = 0
	audioguide.ops.SAVE_TARGET_SEGMENT_LABELS = False
	audioguide.ops.SAVE_CONFIG_FILE = False
	audioguide.ops.TARGET_PATH = audioguide.ops.SOURCE_PATH # look for target files here if not a full path...
	#audioguide.ops.SEGMENTATION_RUN = True
	ag.init(printConfig=False)
	print("\tCreating segmentation label file for %s..."%file)
	print("\tThreshold: %f\t\trise: %f\tRelease (dB above min dB): %f"%(options.THRESHOLD, options.RISE, options.DB))
	ag.loadTarget()
	if options.OUTPUT_FILE == '':
		segFile = audioguide.tgt.filename+'.txt'
	else:
		segFile = os.path.abspath(options.OUTPUT_FILE)
	if options.GRAIN_SIZE == None and options.grainRange == None: 
		audioguide.tgt.writeSegLabelsToDisk(segFile, audioguide.tgt.segs, audioguide.ops.TEMPO_CHANGE)
		SEGS = audioguide.tgt.segs
		print("\n\tAudioGuide found %i segments\n"%len(audioguide.tgt.segs))
	elif type(options.GRAIN_SIZE) in [float, int]:
		SEGS = []
		print("\tSilcing target segments into grains of %f seconds with an overlap of %f"%(options.GRAIN_SIZE, options.GRAIN_OVERLAP))
		out = open(segFile, 'w')
		numbSegs = 0
		for a in audioguide.tgt.segs:
			start = a[0]
			end = start+a[1]
			if options.exhaustive:
				out.write( str(start)+'\t'+str(end) +'\n') # append real segment
				SEGS.append((start,end-start))
				numbSegs += 1

			#print start, end
			while end > start:	
				#print options.exhaustive
				for multi in [2, 3, 4]:
					if options.GRAIN_SIZE*multi <= end:
						#print '\t\t', start+(options.GRAIN_SIZE*multi), end
						if options.exhaustive:
							out.write( str(start)+'\t'+str(start+(options.GRAIN_SIZE*multi)) +'\n')
							SEGS.append((start,end))
							numbSegs += 1
			#print '\t', start, start+options.GRAIN_SIZE
				out.write( str(start)+'\t'+str(start+options.GRAIN_SIZE) +'\n')
				start += options.GRAIN_SIZE/float(options.GRAIN_OVERLAP)
				numbSegs += 1
		print("\n\tAudioGuide found %i grain-segments\n"%numbSegs)
		out.close()
	else:
		out = open(segFile, 'w')
		range = eval(options.grainRange)
		time = 0.
		while True:
			dur = random.uniform(range[0],range[1])
			print(time, "for", dur)
			if time+dur > audioguide.tgt.endSec: break
			out.write( str(time)+'\t'+str(time+dur) +'\n')
			time += dur
		out.close()
	triggers = []
	for key, val in audioguide.tgt.triggerData.iteritems():
		if key == 'totalWeights': continue
		triggers.append('"'+key+'": '+audioguide.utilities.trunc(val['initMax'], 2))
	#print 'TARGET_ONSET_FORCE_MAX = {'+', '.join(triggers)+'}\n'
	
#	durations = []
#	for seg in SEGS: durations.append(seg[1])
#	print durations
